#!/usr/bin/env python3
import argparse
from os import walk
from sys import argv


def has_ending(endings, file_names):
    return any([e in endings for e in [n.split(".")[-1] for n in file_names]])


def is_included(snapshots, dir_path):
    return snapshots or not dir_path.upper().endswith("-SNAPSHOT")


def get_artifacts(repo_dir, endings=("pom", "jar"), snapshots=False):
    return sorted(
        [
            dir_path[len(repo_dir) + 1 :]
            for dir_path, dir_names, file_names in walk(repo_dir)
            if has_ending(endings, file_names) and is_included(snapshots, dir_path)
        ]
    )


def main(repo_dir, endings=("pom", "jar"), snapshots=False):
    for artifact in get_artifacts(repo_dir, endings, snapshots):
        split = artifact.split("/")
        # <groupId>se.lf.csl</groupId>
        # <artifactId>xml.ws</artifactId>
        # <version>1.0.0-SNAPSHOT</version>

        print(
            f"""<dependency>
    <groupId>{".".join(split[:-2])}</groupId>
    <artifactId>{split[-2]}</artifactId>
    <version>{split[-1]}</version>
</dependency>"""
        )

    # print(
    #     f'groupId: {".".join(split[:-2])},',
    #     f'artifactId: {split[-2]},',
    #     f'version: {split[-1]}',
    # )


def parse_args(args):
    parser = argparse.ArgumentParser(description="List artifacts in repository")
    parser.add_argument(
        "repo_dir",
        help="Repository directory",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from glob import glob
from os import environ, makedirs, remove
from os.path import exists, normpath, realpath
from shutil import rmtree
from sys import argv
from traceback import print_exc

from vang.pio.shell import run_command


def get_jars(root):
    return [realpath(p) for p in glob(normpath(f"{root}/**/*.jar"), recursive=True)]


def print_release_info(release_info, develop_branch, release_branch, release_tag):
    content = release_info.split("\n")
    if develop_branch:
        print(content[0])
    if release_branch:
        print(content[1])
    if release_tag:
        print(content[2])


def get_release_info(war, develop_branch, release_branch, release_tag):
    work_dir = environ["HOME"] + "/tmp_wildcat"
    try:
        release_info = "META-INF/release.info"
        release_info_path = f"{work_dir}/{release_info}"
        makedirs(work_dir)
        run_command(f"jar xvf {realpath(war)}", cwd=work_dir, return_output=True)
        with open(
            f"{work_dir}/WEB-INF/classes/{release_info}", "rt", encoding="utf-8"
        ) as f:
            print_release_info(f.read(), develop_branch, release_branch, release_tag)

        for jar in get_jars(work_dir):
            if exists(release_info_path):
                remove(release_info_path)
            run_command(
                f"jar xvf {jar} -x {release_info}", cwd=work_dir, return_output=True
            )
            if exists(release_info_path):
                with open(release_info_path, "rt", encoding="utf-8") as f:
                    print_release_info(
                        f.read(), develop_branch, release_branch, release_tag
                    )
    except OSError:
        print_exc()
    finally:
        rmtree(work_dir, ignore_errors=True)


def main(war, develop_branch, release_branch, release_tag):
    get_release_info(war, develop_branch, release_branch, release_tag)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Print release.info of all artifacts in war"
    )
    parser.add_argument("war", help="The war file")
    parser.add_argument(
        "-d",
        "--develop_branch",
        help="Print clone commands of develop_branch",
        action="store_true",
    )
    parser.add_argument(
        "-r",
        "--release_branch",
        help="Print clone commands of release_branch",
        action="store_true",
    )
    parser.add_argument(
        "-t",
        "--release_tag",
        help="Print clone commands of release_tag",
        action="store_true",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    the_args = parse_args(argv[1:])
    params = [the_args.develop_branch, the_args.release_branch, the_args.release_tag]
    main(the_args.war, *params if any(params) else [True, True, True])



#!/usr/bin/env python3

import argparse
from os import environ
from sys import argv

from vang.jenkins.api import call


def delete_jobs(
    names,
    url=environ.get("JENKINS_REST_URL", None),
    username=environ.get("JENKINS_USERNAME", None),
    password=environ.get("JENKINS_PASSWORD", None),
    verify_certificate=not environ.get("JENKINS_IGNORE_CERTIFICATE", None),
):
    return [
        (
            job_name,
            call(
                f"/job/{job_name}/doDelete",
                method="POST",
                only_response_code=True,
                rest_url=url,
                username=username,
                password=password,
                verify_certificate=verify_certificate,
            ),
        )
        for job_name in names
    ]


def parse_args(args):
    parser = argparse.ArgumentParser(description="Delete Jenkins jobs")
    parser.add_argument(
        "job_names",
        nargs="+",
        help="Jenkins job names",
    )
    return parser.parse_args(args)


def main(job_names):
    for a_job_name, a_response_code in delete_jobs(job_names):
        print(a_job_name, a_response_code)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import logging
from json import loads
from os import name as os_name, system, environ
from os.path import basename
from sys import argv
from typing import Dict, Tuple

from requests import post

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = "https://dev.azure.com"


def create_repo(
    token: str,
    organization: str,
    project: str,
    repo: str,
    verify_certificate: bool = True,
    api_version: str = "6.1-preview.1",
) -> Tuple[int, Dict]:
    url = f"{base_url}/{organization}/{project}/_apis/git/repositories?api-version={api_version}"
    params = {
        "json": {"name": repo},
        "url": url,
        "auth": ("", token),
        "verify": verify_certificate,
    }
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = post(**params)
    logger.info(f"response.status_code: {response.status_code}")
    logger.info(f"response.text: {response.text}")
    response.raise_for_status()
    return response.status_code, loads(response.text)


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(description="Create a repo")
    parser.add_argument(
        "--token",
        default=environ.get("AZDO_TOKEN", ""),
        help="The Azure DevOps authorisation token",
    )
    parser.add_argument(
        "--organisation",
        default=environ.get("AZDO_ORGANISATION", ""),
        help="The Azure DevOps organisation",
    )
    parser.add_argument(
        "--project",
        default=environ.get("AZDO_PROJECT", ""),
        help="The Azure DevOps project",
    )
    parser.add_argument("repo", help="The Azure DevOps repo to create")
    parser.add_argument(
        "-au",
        "--azure_devops_url",
        default="https://dev.azure.com",
        help="The Azure DevOps REST API base url",
    )

    return parser.parse_args(args)


def main(
    token: str, organisation: str, project: str, repo: str, azure_devops_url: str
) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    status_code, repo = create_repo(token, organisation, project, repo)
    print(status_code)

    commands = (
        f'    git remote add origin {repo["remoteUrl"]}\n'
        "    git push -u origin 1.0.x"
    )
    print(
        "If you already have code ready to be pushed to this repository "
        "then run this in your terminal."
    )
    print(commands)
    if os_name == "posix":
        system(f'echo "{commands}\c" | pbcopy')
        print("(The commands are copied to the clipboard)")


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from glob import glob
from os import environ
from os.path import realpath, normpath
from pathlib import Path
from sys import argv

from requests import post

from vang.core.core import pmap_unordered
from vang.jenkins.download_configs import get_all_job_names


def get_default_jenkins_spec():
    return {
        "url": environ.get("JENKINS_REST_URL", None),
        "username": environ.get("JENKINS_USERNAME", None),
        "password": environ.get("JENKINS_PASSWORD", None),
        "verify_certificate": not environ.get("JENKINS_IGNORE_CERTIFICATE", None),
    }


def create_params(jenkins_spec):
    return {
        "auth": (jenkins_spec["username"], jenkins_spec["password"]),
        "verify": jenkins_spec["verify_certificate"],
    }


def post_config(job_name, config, update, jenkins_spec):
    update_uri = f"job/{job_name}/config.xml"
    create_uri = f"createItem?name={job_name}"

    params = create_params(jenkins_spec)
    params["url"] = f'{jenkins_spec["url"]}/{update_uri if update else create_uri}'
    params["data"] = config
    params["headers"] = {"Content-Type": "application/xml"}

    response = post(**params)
    response.raise_for_status()
    return response.status_code, response.text


def get_input_file(input_dir, job_name):
    return f"{input_dir}/{job_name}/config.xml"


def read(file_path):  # pragma: no cover
    with open(file_path, "rt", encoding="utf-8") as f:
        return f.read()


def get_configs_paths(input_dir):
    return [
        Path(normpath(realpath(p)))
        for p in glob(f"{input_dir}/**/config.xml", recursive=True)
    ]


def upload_config(config_path, existing_job_names, jenkins_spec):
    job_name = config_path.parent.name
    try:
        config = config_path.read_text("utf-8")
        post_config(job_name, config, job_name in existing_job_names, jenkins_spec)
        return job_name
    except IOError as e:
        return f"{job_name}: {e}"


def upload_configs(
    input_dir, jenkins_spec=get_default_jenkins_spec(), max_processes=10
):
    existing_job_names = get_all_job_names(jenkins_spec)
    yield from pmap_unordered(
        lambda config_path: upload_config(
            config_path, existing_job_names, jenkins_spec
        ),
        get_configs_paths(input_dir),
        max_processes,
    )


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Upload Jenkins configs, i.e. update existing jobs and create non existing"
    )
    parser.add_argument("input_dir", help="The dir to load configs from.")
    return parser.parse_args(args)


def main(input_dir):
    for job_name in upload_configs(input_dir, get_default_jenkins_spec()):
        print(job_name)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.tfs.api import call
from vang.tfs.get_projects import get_projects


def get_repos(
    organisations=None, projects=None, names=False, repo_specs=False, urls=False
):
    if organisations:
        projects = get_projects(organisations, project_specs=True)
    if not projects:
        return []
    repos = [
        (project, repo)
        for project in projects
        for repo in call(f"/{project}/_apis/git/repositories?api-version=3.2")["value"]
    ]
    if names:
        return [repo[1]["name"] for repo in repos]
    if repo_specs:
        return [f'{repo[0]}/{repo[1]["name"]}' for repo in repos]
    if urls:
        return [repo[1]["remoteUrl"] for repo in repos]
    return repos


def main(organisations, projects, names, repo_specs, urls):
    for repo in get_repos(organisations, projects, names, repo_specs, urls):
        print(repo)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Get TFS repos")
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        "-o", "--organisations", nargs="+", help="TFS organisations, e.g organisation"
    )
    required_group.add_argument(
        "-p", "--projects", nargs="+", help="TFS projects, e.g organisation/project"
    )

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        "-n", "--names", action="store_true", help="Get only repo names"
    )
    optional_group.add_argument(
        "-r",
        "--repo_specs",
        help="Print only organisation/project/name",
        action="store_true",
    )
    optional_group.add_argument(
        "-u", "--urls", action="store_true", help="Get only repo urls"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from glob import glob
from itertools import product
from os import PathLike
from os.path import normpath, realpath
from pathlib import Path
from subprocess import PIPE, STDOUT, run, CompletedProcess
from sys import argv
from typing import Iterable

from vang.pio.shell import run_commands


def get_work_dirs(find: str, root: str) -> Iterable[PathLike]:
    return [
        Path(realpath(p)).parent
        for p in glob(
            normpath(f"{root}/**/{find}"),
            recursive=True,
        )
    ]


def get_command(commands: Iterable[str]) -> list[str]:
    return [" && ".join(commands)]


def execute_in_parallel(
    root: str, commands: Iterable[str], find: str = ".git/"
) -> Iterable[CompletedProcess]:
    commands_and_work_dirs = tuple(
        product(
            get_command(commands),
            get_work_dirs(find, root),
        )
    )
    yield from run_commands(commands_and_work_dirs, max_processes=25, check=False)


def execute_in_sequence(
    root: str, commands: Iterable[str], find: str = ".git/", timeout: int = None
) -> Iterable[CompletedProcess]:
    command = get_command(commands)[0]
    for cwd in get_work_dirs(find, root):
        yield run(
            command,
            cwd=cwd,
            stdout=PIPE,
            stderr=STDOUT,
            check=False,
            timeout=timeout,
            shell=True,
        )


def main(
    root: str, commands: Iterable[str], find: str = ".git/", sequence: bool = False
):
    execute = execute_in_sequence if sequence else execute_in_parallel
    for process in execute(root, commands, find):
        print(process.stdout.decode(errors="ignore").strip())


def parse_args(args):
    parser = ArgumentParser(
        description="Execute commands recursively (default in git repositories)"
    )
    parser.add_argument("commands", nargs="+", help="Commands")
    parser.add_argument("-r", "--root", help="The root directory", default=".")
    parser.add_argument(
        "-f",
        "--find",
        help="The file/dir to be in the directory in which to execute, "
        'default ".git/"',
        default=".git/",
    )
    parser.add_argument(
        "-s",
        "--sequence",
        help="Run the commands in sequence, i.e. not in parallel",
        action="store_true",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from os import walk, rename, makedirs, listdir, rmdir
from os.path import join, sep
from sys import argv
from typing import Iterable, Callable

from vang.pio.rsr import _replace_in_file, _in


def file_content_replace_function(line: str, old: str, new: str) -> str:
    line = line.replace(old, new)
    line = line.replace(old.replace(".", sep), new.replace(".", sep))

    # line = line.replace(f'<groupId>{old}</groupId>', f'<groupId>{new}</groupId>')
    # line = line.replace(f'"{old}', f'"{new}')
    # line = line.replace('{' + old, '{' + new)
    # line = line.replace('<' + old, '<' + new)
    # line = line.replace('=' + old, '=' + new)
    # line = line.replace("='" + old, "='" + new)
    # line = line.replace(old.replace('.', sep), new.replace('.', sep))
    # line = sub(f'^package +{old}', f'package {new}', line)
    # line = sub(f'^import +{old}', f'import {new}', line)
    # line = sub(f'^import +static +{old}', f'import static {new}', line)
    return line


def file_path_replace_function(file: str, old: str, new: str) -> str:
    return file.replace(old.replace(".", sep), new.replace(".", sep))


def _replace_file(
    old: str,
    new: str,
    path: str,
    file: str,
    replace_function: Callable[[str, str, str], str],
) -> None:
    new_path = replace_function(path, old, new)
    if new_path != path:
        makedirs(new_path, exist_ok=True)
        rename(join(path, file), join(new_path, file))


def _regroup(root: str, excludes: Iterable[str], old: str, new: str) -> None:
    for dir_path, dir_names, files in walk(root, False):
        if not any(_in(d, excludes) for d in dir_path.split(sep)):
            for file in files:
                if not _in(file, excludes):
                    _replace_in_file(
                        old, new, join(dir_path, file), file_content_replace_function
                    )
                    _replace_file(old, new, dir_path, file, file_path_replace_function)

            for dir_name in dir_names:
                if not listdir(join(dir_path, dir_name)):
                    rmdir(join(dir_path, dir_name))


def regroup(old: str, new: str, dirs: Iterable[str]) -> None:
    for d in dirs:
        _regroup(d, [".git", ".gitignore", "target"], old, new)


def main(old: str, new: str, dirs: Iterable[str]) -> None:
    regroup(old, new, dirs)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Change group for maven module, including package, all its imports and path references"
    )
    parser.add_argument("old", help="old group")
    parser.add_argument("new", help="new group")
    parser.add_argument("-d", "--dirs", nargs="*", default=["."])
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.get_branches import get_branches
from vang.bitbucket.get_projects import get_projects
from vang.bitbucket.get_repos import get_repos
from vang.bitbucket.get_tags import get_all_tags
from vang.core.core import pmap_unordered


def validate_repos(project):
    valid = []
    corrupt = []
    for r in get_repos(project):
        s = "/".join(r)
        try:
            get_branches(r)
            get_all_tags(r)
            valid.append(s)
        except KeyError:
            corrupt.append(s)
    return valid, corrupt


def validate_projects(max_projects=None, max_processes=10):
    for valid, corrupt in pmap_unordered(
        lambda p: validate_repos(p["key"]),
        get_projects(),
        processes=max_processes,
    ):
        for v in valid:
            print(f"{v} is OK")
        for c in corrupt:
            print(f"{c} is CORRUPT")


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Validate projects by checking that the rest api can be used to retrieve repos, branches and tags."
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    validate_projects(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from os import environ
from sys import argv

from requests import put


def read_file(file_path):  # pragma: no cover
    with open(file_path, "rb") as f:
        return f.read()


def upload(file_path, repository, repository_path, url, username, password):
    url = f"{url}/repository/{repository}/{repository_path}"
    data = read_file(file_path)
    headers = {"Content-Type": "application/octet-stream"}
    response = put(url, data=data, headers=headers, auth=(username, password))
    if response.status_code != 201:
        raise OSError(f"{response.status_code}, {response.content}")
    return response.status_code


def parse_args(args):
    parser = ArgumentParser(description="Get assets")
    parser.add_argument("file_path", help="File to upload, e.g. ./myartifact-1.0.0.jar")
    parser.add_argument("repository", help="Nexus3 repository, e.g. maven-releases")
    parser.add_argument(
        "repository_path",
        help="Path within Nexus3 repository, e.g com/myorg/myartifact/1.0.0/myartifact-1.0.0.jar",
    )
    parser.add_argument(
        "-l",
        "--url",
        default=environ.get("NEXUS3_REST_URL", None),
        help="Nexus3 url, e.g. http://nexus_host:8080",
    )
    parser.add_argument(
        "-u",
        "--username",
        default=environ.get("NEXUS3_USERNAME", None),
        help="Nexus3 username",
    )
    parser.add_argument(
        "-p",
        "--password",
        default=environ.get("NEXUS3_PASSWORD", None),
        help="Nexus3 password",
    )
    return parser.parse_args(args)


def main(file_path, repository, repository_path, url, username, password):
    print(upload(file_path, repository, repository_path, url, username, password))


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from os import environ

from requests import delete, get, post, put


def call(
    uri,
    request_data=None,
    method="GET",
    only_response_code=False,
    rest_url=environ.get("TFS_REST_URL", None),
    token=environ.get("TFS_TOKEN", None),
):
    """Makes a REST call to TFS rest api.
    May use three environment variables:
    * TFS_REST_URL, e.g. http://myorg.com/stash
    * TFS_TOKEN, the tfs token

    Args:
        uri (str): e.g. "{organisation}/{project}/_apis/build/definitions/{definition_id}?api-version=3.2"
        request_data (dict): the JSON request
        method: http method
        only_response_code: default False
        rest_url: default environ.get('TFS_REST_URL', None)
        token: default environ.get('TFS_TOKEN', None),

    Return:
          the JSON response
    """
    return call_url(f"{rest_url}{uri}", request_data, method, only_response_code, token)


def call_url(
    url,
    request_data=None,
    method="GET",
    only_response_code=False,
    token=environ.get("TFS_TOKEN", None),
    verify_certificate=False,
):
    """Makes a REST call to TFS rest api.
    May use three environment variables:
    * TFS_TOKEN, the tfs token

    Args:
        url (str): e.g. "http://myorg:8080/tfs/{organisation}/{project}/_apis/build/definitions/{definition_id}?api-version=3.2"
        request_data (dict): the JSON request
        method: http method
        only_response_code: default False
        token: default environ.get('TFS_TOKEN', None),
        verify_certificate: True if https certificate should be verified,

    Return:
          the JSON response
    """
    m = {
        "DELETE": delete,
        "GET": get,
        "POST": post,
        "PUT": put,
    }[method]

    params = {"url": url, "auth": ("", token), "verify": verify_certificate}
    if request_data:
        params["json"] = request_data

    response = m(**params)
    return (
        response.status_code
        if only_response_code
        else response.json()
        if response.text
        else response.status_code()
    )



#!/usr/bin/env python3
import argparse
from os import environ, name
from os.path import exists
from sys import argv

from vang.pio.shell import run_command


def switch_settings(ending):
    source = f'{environ["HOME"]}/.m2/settings_{ending}.xml'
    if not exists(source):
        raise ValueError(f"{source} does not exist")
    target = f'{environ["HOME"]}/.m2/settings.xml'
    return run_command(f"ln -sf {source} {target}", True)


def main(ending):
    if name == "posix":
        rc, output = switch_settings(ending)
        print(output)
    else:
        print("Platform not supported. Please implement, and make a pull request.")


def parse_args(args):
    parser = argparse.ArgumentParser(description="Switch maven settings in ~/.m2")
    parser.add_argument(
        "ending",
        help="The ending of the settings file to switch to"
        ", e.g. FOO for ~/.m2/settings_FOO.xml",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import environ, makedirs
from sys import argv

from requests import get

from vang.core.core import pmap_unordered


def get_default_jenkins_spec():
    return {
        "url": environ.get("JENKINS_REST_URL", None),
        "username": environ.get("JENKINS_USERNAME", None),
        "password": environ.get("JENKINS_PASSWORD", None),
        "verify_certificate": not environ.get("JENKINS_IGNORE_CERTIFICATE", None),
    }


def create_params(jenkins_spec):
    return {
        "auth": (jenkins_spec["username"], jenkins_spec["password"]),
        "verify": jenkins_spec["verify_certificate"],
    }


def get_config(job_name, jenkins_spec):
    params = create_params(jenkins_spec)
    params["url"] = f'{jenkins_spec["url"]}/job/{job_name}/config.xml'
    response = get(**params)
    response.raise_for_status()
    return response.status_code, response.text


def get_output_file(output_dir, job_name):
    return f"{output_dir}/{job_name}/config.xml"


def save(output_file, config):
    with open(output_file, "wt", encoding="utf-8") as f:
        f.write(config)


def download_config(output_dir, job_name, jenkins_spec):
    try:
        makedirs(f"{output_dir}/{job_name}", exist_ok=True)
        save(
            get_output_file(output_dir, job_name), get_config(job_name, jenkins_spec)[1]
        )
        return job_name
    except IOError as e:
        return f"{job_name}: {e}"


def download_configs(
    output_dir, job_names, jenkins_spec=get_default_jenkins_spec(), max_processes=10
):
    yield from pmap_unordered(
        lambda job_name: download_config(output_dir, job_name, jenkins_spec),
        job_names,
        max_processes,
    )


def get_all_job_names(jenkins_spec=get_default_jenkins_spec(), max_processes=10):
    params = create_params(jenkins_spec)
    params["url"] = f'{jenkins_spec["url"]}/api/json'
    response = get(**params)
    response.raise_for_status()
    return [job["name"] for job in response.json()["jobs"]]


def parse_args(args):
    parser = argparse.ArgumentParser(description="Download Jenkins configs")
    parser.add_argument("output_dir", help="The dir to save configs in")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-n",
        "--job_names",
        nargs="+",
        help="Name of the jobs for which to download configs",
    )
    group.add_argument("-a", "--all", action="store_true", help="Download all configs")
    return parser.parse_args(args)


def main(output_dir, job_names, all):
    for job_name in download_configs(
        output_dir, get_all_job_names() if all else job_names
    ):
        print(job_name)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from base64 import encodebytes
from os import name, system
from sys import argv


def get_basic_auth(username, password):
    """Returns basic authentication.

    Args:
        username (str): the username (defaults to environment variable 'U')
        password (str): the password (defaults to environment variable 'P')

    Return:
          base 64 encoded Authorization header value

    >>> get_basic_auth("foo", "bar")
    'Basic Zm9vOmJhcg=='
    """

    auth = f"{username}:{password}"
    return f"Basic {encodebytes(auth.encode()).decode('UTF-8').strip()}"


def get_basic_auth_header(username, password):
    return f"Authorization: {get_basic_auth(username, password)}"


def parse_args(args):
    parser = ArgumentParser(
        description="Prints and place in clipboard basic authentication header"
    )
    parser.add_argument("username", help="Username")
    parser.add_argument("password", help="Password")
    return parser.parse_args(args)


def main(username, password):
    basic_auth_header = get_basic_auth_header(username, password)
    if name == "posix":
        system(f"echo '{basic_auth_header}\\c' | pbcopy")
        print(f"'{basic_auth_header}' copied to clipboard")
    else:
        print(basic_auth_header)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from shutil import copy2
from sys import argv

from vang.misc.mac_ip import get_ip_address

etc_hosts_file = "/etc/hosts"
encoding = "utf-8"


def update(backup_file=None):
    if backup_file:
        copy2(etc_hosts_file, backup_file)

    with open(etc_hosts_file, "rt", encoding=encoding) as f:
        original_lines = f.readlines()

    if any(["ext.local" in l for l in original_lines]):
        updated_lines = [
            f"{get_ip_address()}   ext.local\n" if "ext.local" in l else l
            for l in original_lines
        ]
    else:
        updated_lines = list(original_lines)
        if not updated_lines[-1].endswith("\n"):
            updated_lines[-1] = updated_lines[-1] + "\n"
        updated_lines.append(f"{get_ip_address()}   ext.local\n")

    if updated_lines != original_lines:
        with open(etc_hosts_file, "wt", encoding=encoding) as f:
            f.writelines(updated_lines)

    return updated_lines


def parse_args(args):
    parser = ArgumentParser(
        description="Updates ext.local entry in /etc/hosts to the current ip address."
        "Run it with sudo or give yourself write permission to the etc/hosts "
        "file and run it without sudo."
        "To run it automatically, modify ext_local.plist and copy it, load and "
        "start as below: "
        "sudo cp ext_local.plist /Library/LaunchDaemons/com.github.bjuvensjo.scripts.ext_local.plist, "
        "sudo launchctl load /Library/LaunchDaemons/com.github.bjuvensjo.scripts.ext_local.plist, "
        "sudo launchctl start com.github.bjuvensjo.scripts.ext_local"
    )
    parser.add_argument(
        "-b", "--backup", help="Backup etc/hosts in this file", default=None
    )
    return parser.parse_args(args)


def main(backup):
    print("".join(update(backup)))


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from os import environ
from sys import argv
from xml.etree.ElementTree import fromstring

from requests import get


def get_metadata(root_url, username, password):
    response = get(f"{root_url}/maven-metadata.xml", auth=(username, password))
    e = fromstring(response.text)
    p = "versioning/snapshotVersions/snapshotVersion"
    return e.findtext(f"{p}/value").strip(), e.findtext(f"{p}/extension").strip()


def download(output_file, url, username, password):
    with open(output_file, "wb") as f:
        f.write(get(url, auth=(username, password)).content)


def main(
    output_dir,
    nexus_url,
    repo,
    group_id,
    artifact_id,
    version,
    username=environ.get("NEXUS3_USERNAME", None),
    password=environ.get("NEXUS3_PASSWORD", None),
):
    root_url = (
        f'{nexus_url}/{repo}/{group_id.replace(".", "/")}/{artifact_id}/{version}'
    )
    snapshot_version, extension = get_metadata(root_url, username, password)
    output_file = f"{output_dir}/{artifact_id}-{version}.{extension}"
    url = f"{root_url}/{artifact_id}-{snapshot_version}.{extension}"
    print(f"Downloading: {url} to {output_file}")
    download(output_file, url, username, password)
    print(f"Downloaded: {url} to {output_file}")


def parse_args(args):
    parser = ArgumentParser(
        description="Download is_later snapshot from Nexus (requires: pipenv install requests)"
    )
    parser.add_argument("output_dir", help="output dir")
    parser.add_argument(
        "nexus_url", help="Nexus url, e.g. http://localhost:9002/repository"
    )
    parser.add_argument("repo", help="Nexus repository, e.g. maven-snapshots")
    parser.add_argument("group_id", help="Maven group id")
    parser.add_argument("artifact_id", help="Maven artifact id")
    parser.add_argument("version", help="Maven version")
    parser.add_argument("-u", "--username", help="username for basic auth")
    parser.add_argument("-p", "--password", help="password for basic auth")
    return parser.parse_args(args)


if __name__ == "__main__":
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import logging
import re
from sys import argv, stdout

from vang.pio.shell import run_command

logging.basicConfig(stream=stdout, level=logging.INFO)


def get_dependencies(dependency_tree_split):
    lines = dependency_tree_split.split("\n")
    module = lines[0].replace("[INFO]", "").strip()
    direct = []
    transitive = []
    for line in lines[1:]:
        if line.startswith("[INFO] +- ") or line.startswith("[INFO] \- "):
            direct.append(line.split("-", maxsplit=1)[1].strip())
        else:
            transitive.append(line.split("-", maxsplit=1)[1].strip())
    return module, sorted(direct), sorted(transitive)


def split_dependency_tree(dependency_tree):
    """Splits a multi module into separate parts and a single module to one."""
    splits = re.split(r"\[INFO\] --- maven-dependency-plugin.*", dependency_tree)
    if len(splits) < 2:
        return []
    modules = []
    for m in splits[1:]:
        lines = []
        for line in m.strip().split("\n"):
            if len(line.split(":")) >= 4:
                lines.append(line)
            else:
                break
        modules.append("\n".join(lines))

    return modules


def get_dependency_tree(pom_file):
    code, output = run_command(f"mvn dependency:tree -f {pom_file}", True)
    return output


def get_modules_dependencies(pom_file):
    dependency_tree = get_dependency_tree(pom_file)
    logging.debug("dependency tree: %s", dependency_tree)
    modules_dependency_trees = split_dependency_tree(dependency_tree)
    logging.debug("modules dependency tree: %s", modules_dependency_trees)
    modules_dependencies = [get_dependencies(mdt) for mdt in modules_dependency_trees]
    return modules_dependencies


def main(pom_file, only_direct, only_transitive):
    for module, direct, transitive in get_modules_dependencies(pom_file):
        print("#" * 80)
        print(module)
        if not only_transitive:
            print("direct:\n  ", "\n  ".join(direct), sep="")
        if not only_direct:
            print("transitive:\n  ", "\n  ".join(transitive), sep="")
        print("#" * 80)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Get direct and transitive dependencies from provided pom.xml, "
        "e.g. ./get_dependencies.py ./pom.xml. "
        "Depends on Maven."
    )
    parser.add_argument("-p", "--pom_file", help="Path to pom file", default="pom.xml")
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        "-d", "--only_direct", help="Print only direct dependants", action="store_true"
    )
    group.add_argument(
        "-t",
        "--only_transitive",
        help="Print only transitive dependants",
        action="store_true",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from glob import glob
from os import makedirs
from os.path import exists
from re import match
from sys import argv

from vang.pio.shell import run_command


def is_excluded(py_file):
    return any(
        (
            match(regexp, py_file.split("/")[-1])
            for regexp in ["__init__.py", ".*slask.*", ".*Slask.*"]
        )
    )


def has_main(py_file):
    main_lines = [
        "if __name__ == '__main__':",
        'if __name__ == "__main__":',
    ]
    with open(py_file, "rt", encoding="utf-8") as f:
        for line in f:
            # if line and match(r'.*if __name__ == ("|\')__main__("|\'): *',
            #                   line):
            print(line)
            if line.strip() in main_lines:
                return True
    return False


def map_to_link_name(main_file):
    main_file_package, main_file_name = main_file.split("/")[-2:]
    return f'{main_file_package}-{main_file_name[:-3].replace("_", "-")}'


def create_symlinks(source, target):
    if not exists(target):
        makedirs(target)

    for main_file in [
        f
        for f in glob(f"{source}/**/*.py", recursive=True)
        if has_main(f) and not is_excluded(f)
    ]:
        link_file = f"{target}/{map_to_link_name(main_file)}"
        if exists(link_file):
            print(f"{link_file} already exists")
        else:
            command = f"ln -s {main_file} {link_file}"
            print(command)
            run_command(command)


def parse_args(args):
    parser = ArgumentParser(description="Create symlinks")
    parser.add_argument("source", help="Source root")
    parser.add_argument("target", help="Target")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    create_symlinks(**parse_args(argv[1:]).__dict__)



from vang.artifactory.mirror import (
    not_folders,
    not_index,
    not_maven_metadata,
    latest_version,
)

work_dir = "./work_dir"
# Spec of the artifactory instance to copy from
from_artifactory_spec = {
    "url": "<from_artifactory_url>",
    "username": "<username>",
    "password": "<password>",
}
# Spec of the artifactory instance to copy to
# In example below, by creating a copy of from_artifactory_spec with a different url
to_artifactory_spec = dict(from_artifactory_spec, url="<to_artifactory_url>")

# Specify a pair (tuple) if the repo key to mirror to is not the same as the repo key to mirror from
repositories = [("from_repo_key", "to_repo_key"), "from_and_to_repo_key"]

filters = (not_folders, not_index, not_maven_metadata, latest_version)



#!/usr/bin/env python3
from argparse import ArgumentParser
from os import environ, path, walk
from os.path import normpath, exists
from sys import argv
from xml.etree.ElementTree import parse

NAME_SPACES = {"ns": "http://maven.apache.org/SETTINGS/1.0.0"}


def text(e, n):
    return e.findtext(f"ns:{n}", namespaces=NAME_SPACES)


def findall(e, n):
    return e.findall(f".//ns:{n}", namespaces=NAME_SPACES)


def get_settings_info(settings_path):
    e = parse(normpath(settings_path))

    local_repo = text(e, "localRepository")
    if "${user.home}" in local_repo:
        local_repo = local_repo.replace("${user.home}", environ["HOME"])

    remotes = {text(r, "id"): text(r, "url") for r in findall(e, "repository")}

    plugin_remotes = {
        text(r, "id"): text(r, "url") for r in findall(e, "pluginRepository")
    }

    mirrors = {text(r, "id"): text(r, "url") for r in findall(e, "mirror")}

    return local_repo, {**remotes, **plugin_remotes, **mirrors}


def excluded(f):
    if f in [
        "_remote.repositories",
        "maven-metadata-local.xml",
        "maven-metadata-ossrh.xml",
        "resolver-status.properties",
    ]:
        return True
    if f.endswith("asc"):
        return True
    if f.endswith("lastUpdated"):
        return True
    return False


def get_file_repos_info(local_repo, files):
    for f in files:
        root, file = f.rsplit("/", 1)
        remote_repositories = path.join(root, "_remote.repositories")
        if exists(remote_repositories):
            yield remote_repositories, root[len(local_repo) :], (file,)


def get_repos_info(local_repo):
    for root, dirs, files in walk(local_repo):
        if files and "_remote.repositories" in files:
            yield (
                path.join(root, "_remote.repositories"),
                root[len(local_repo) :],
                [f for f in files if not excluded(f)],
            )


def parse_remote_repos(remote_repositories, remotes):
    with open(remote_repositories, "rt", encoding="utf-8") as f:
        lines = [l for l in f.read().splitlines() if not l.startswith("#")]
        return {
            k: remotes.get(v.replace("=", ""), None)
            for k, v in [l.split(">") for l in lines]
        }


def create_commands(remote_dict, artifact_path, artifacts):
    def create_artifact_command(artifact):
        extension = artifact.rsplit(".", 1)[1]
        remote_dict_key = (
            artifact[: -(len(extension) + 1)]
            if extension in ["sha1", "md5", "sha256"]
            else artifact
        )

        remote_url = remote_dict.get(remote_dict_key, None)
        if remote_url:
            if remote_url.endswith("/"):
                remote_url = remote_url[:-1]
            rel_path = f"{artifact_path}/{artifact}"
            return f'curl -o "{rel_path}" {remote_url}/{rel_path}'
        else:
            return None

    mkdir_commands = [f'mkdir -p "{artifact_path}"']
    download_commands = [create_artifact_command(artifact) for artifact in artifacts]
    return mkdir_commands + download_commands if any(download_commands) else []


def create_urls(remote_dict, artifact_path, artifacts):
    def create_artifact_urls(artifact):
        extension = artifact.rsplit(".", 1)[1]
        remote_dict_key = (
            artifact[: -(len(extension) + 1)]
            if extension in ["sha1", "md5", "sha256"]
            else artifact
        )

        remote_url = remote_dict.get(remote_dict_key, None)
        if remote_url:
            if remote_url.endswith("/"):
                remote_url = remote_url[:-1]
            rel_path = f"{artifact_path}/{artifact}"
            return f"{remote_url}/{rel_path}"
        else:
            return f"No url found for: {artifact_path}/{artifact}"

    urls = [create_artifact_urls(artifact) for artifact in artifacts]
    return urls if any(urls) else []


def main(settings_file, urls_only, skipped, files=None):
    skipped_artifacts = []
    local_repo, remotes = get_settings_info(settings_file)

    repos_info = (
        get_file_repos_info(local_repo, files) if files else get_repos_info(local_repo)
    )
    for remote_repos, artifact_path, artifacts in repos_info:
        remote_dict = parse_remote_repos(remote_repos, remotes)

        params = [remote_dict, artifact_path, artifacts]
        outputs = create_urls(*params) if urls_only else create_commands(*params)

        if outputs:
            for c in outputs:
                print(c)
        else:
            skipped_artifacts.append(artifact_path)

    if skipped:
        for s in skipped_artifacts:
            print("# Skipped:", s)


def parse_args(args):
    parser = ArgumentParser(
        description="Print download commands (POSIX) to enable making "
        "a remote repo based on the content of a local"
    )
    parser.add_argument("settings_file", help="Settings file")
    parser.add_argument(
        "-u", "--urls_only", help="Print only urls", action="store_true"
    )
    parser.add_argument(
        "-s", "--skipped", help="Print skipped artifacts", action="store_true"
    )
    parser.add_argument(
        "-f",
        "--files",
        nargs="*",
        help="Only for specified files and not all in local_repo",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from argparse import ArgumentParser
from os import environ
from pprint import pprint
from sys import argv

from requests import get


def get_assets_page(repository, url, username, password, continuation_token=None):
    token_param = (
        f"&continuationToken={continuation_token}" if continuation_token else ""
    )
    url = f"{url}/service/rest/v1/assets?repository={repository}{token_param}"
    response = get(url, auth=(username, password))
    if response.status_code != 200:
        raise OSError(f"{response.status_code}, {response.content}")
    return response.json()


def get_assets(repository, url, username, password):
    assets_page = get_assets_page(repository, url, username, password)
    continuation_token = assets_page["continuationToken"]
    items = assets_page["items"]
    while continuation_token:
        assets_page = get_assets_page(
            repository, url, username, password, continuation_token
        )
        continuation_token = assets_page["continuationToken"]
        items += assets_page["items"]
    return items


def parse_args(args):
    parser = ArgumentParser(description="Get assets")
    parser.add_argument("repository", help="Nexus3 repository, e.g. maven-releases")
    parser.add_argument(
        "-l",
        "--url",
        default=environ.get("NEXUS3_REST_URL", None),
        help="Nexus3 url, e.g. http://nexus_host:8080",
    )
    parser.add_argument(
        "-u",
        "--username",
        default=environ.get("NEXUS3_USERNAME", None),
        help="Nexus3 username",
    )
    parser.add_argument(
        "-p",
        "--password",
        default=environ.get("NEXUS3_PASSWORD", None),
        help="Nexus3 password",
    )
    return parser.parse_args(args)


def main(repository, url, username, password):
    pprint(get_assets(repository, url, username, password))


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
""" Makes Maven multi module project. """

from argparse import ArgumentParser
from os import makedirs, getcwd
from os.path import realpath, relpath, dirname, normpath, sep
from sys import argv

import vang.maven.pom as pom

POM_TEMPLATE = """<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>
    <groupId>###group_id###</groupId>
    <artifactId>###artifact_id###</artifactId>
    <version>###version###</version>
    <packaging>pom</packaging>
    <modules>
###modules###
    </modules>
</project>"""


def get_pom(pom_infos, output_dir, group_id, artifact_id, version):
    """Returns multi module pom content for pom_infos
    with paths relative to output_dir."""
    modules = "\n".join(
        "        <module>{}</module>".format(
            relpath(realpath(dirname(info["pom_path"])), realpath(output_dir))
        )
        for info in pom_infos
    )
    return (
        POM_TEMPLATE.replace("###group_id###", group_id)
        .replace("###artifact_id###", artifact_id)
        .replace("###version###", version)
        .replace("###modules###", modules)
    )


def make_project(pom_infos, output_dir, group_id, artifact_id, version, **kwargs):
    """Makes a Maven multi module project."""
    pom = get_pom(pom_infos, output_dir, group_id, artifact_id, version)
    makedirs(output_dir)
    with open(
        normpath(f"{output_dir}/pom.xml"),
        "wt",
        encoding="utf-8",
    ) as pom_file:
        pom_file.write(pom)


def get_pom_infos(source_dir):
    pom_infos = []
    for pom_path in pom.get_pom_paths(source_dir):
        try:
            pom_info = pom.get_pom_info(pom_path)
            pom_infos.append(pom_info)
        except Exception as e:  # pragma: no cover
            print(f"Can not add {pom_path}")
            print(e)
    return pom_infos


def parse_args(args):
    parser = ArgumentParser(description="Create Maven multi module project")
    parser.add_argument(
        "-d", "--use_defaults", action="store_true", help="Create with default values."
    )
    return parser.parse_args(args)


def main(use_defaults):
    artifact_id = f"ws-{getcwd().split(sep)[-1]}"
    defaults = {
        "group_id": "my.group",
        "artifact_id": artifact_id,
        "version": "1.0.0-SNAPSHOT",
        "source_dir": ".",
        "output_dir": artifact_id,
    }

    if use_defaults:
        pom_infos = get_pom_infos(defaults["source_dir"])
        make_project(pom_infos, **defaults)
    else:

        group_id = str(
            input(f'groupId (default {defaults["group_id"]}): ') or defaults["group_id"]
        )
        artifact_id = str(
            input(f'artifactId (default {defaults["artifact_id"]}): ')
            or defaults["artifact_id"]
        )
        version = str(
            input(f'version (default {defaults["version"]}): ') or defaults["version"]
        )
        source_dir = normpath(
            str(
                input(f'sourceDir: (default {defaults["source_dir"]})')
                or defaults["source_dir"]
            )
        )
        output_dir = normpath(
            str(
                input(f'outputDir: (default {defaults["output_dir"]})')
                or defaults["output_dir"]
            )
        )

        pom_infos = get_pom_infos(source_dir)
        make_project(pom_infos, output_dir, group_id, artifact_id, version)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from json import dumps
from sys import argv

from vang.tfs.api import call_url
from vang.tfs.list_build_definitions import list_build_definitions


def get_build_definitions(
    organisations=None,
    projects=None,
    filter_name=None,
):
    build_definitions_dict = list_build_definitions(
        organisations, projects, filter_name
    )
    return {
        name: call_url(definition["url"])
        for name, definition in build_definitions_dict.items()
    }


def main(organisations, projects, filter_name=None, format_output=False):
    for name, definition in get_build_definitions(
        organisations, projects, filter_name
    ).items():
        json = dumps(definition, indent=4) if format_output else dumps(definition)
        print(json)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Get TFS build definitions")
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        "-o", "--organisations", nargs="+", help="TFS organisations, e.g organisation"
    )
    required_group.add_argument(
        "-p", "--projects", nargs="+", help="TFS projects, e.g organisation/project"
    )

    parser.add_argument(
        "-fn",
        "--filter_name",
        help="Filters to definitions whose names equal this value. Append a * to filter to definitions whose names "
        "start with this value",
    )

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        "-f", "--format_output", action="store_true", help="Format output"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from os import name as os_name
from os import system
from sys import argv

from vang.tfs.api import call


def create_repo(repo):
    organisation, project, name = repo.split("/")
    return call(
        f"/{organisation}/{project}/_apis/git/repositories?api-version=3.2",
        request_data={"name": name},
        method="POST",
    )


def main(repo):
    response = create_repo(repo)
    commands = (
        f'    git remote add origin {response["remoteUrl"]}\n'
        "    git push -u origin develop"
    )
    print(
        "If you already have code ready to be pushed to this repository "
        "then run this in your terminal."
    )
    print(commands)
    if os_name == "posix":
        system(f'echo "{commands}\c" | pbcopy')
        print("(The commands are copied to the clipboard)")


def parse_args(args):
    parser = argparse.ArgumentParser(description="Create TFS repository")
    parser.add_argument(
        "repo", help="The TFS repository to create, e.g. organisation/project/repo1"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import hashlib
from itertools import count
from json import dumps
from os import makedirs
from os.path import exists

from requests import get, put

import vang.artifactory.mirror_config as config
from vang.artifactory.diff_repos import diff, get_repo_content
from vang.core.core import pmap_unordered
from vang.misc.version import get_latest_snapshot, get_latest_rc, get_latest_fixed


def read_file(file_path):  # pragma: no cover
    with open(file_path, "rb") as f:
        return f.read()


def get_checksums(the_bytes):
    return (
        hashlib.md5(the_bytes).hexdigest(),
        hashlib.sha1(the_bytes).hexdigest(),
        hashlib.sha256(the_bytes).hexdigest(),
    )


def sha1sum(file_path):
    return get_checksums(read_file(file_path))[1]


def download_file(output_file, url, sha1, retry=True):
    if exists(output_file) and sha1sum(output_file) == sha1:
        return True, "Already downloaded"
    with open(output_file, "wb") as f:
        result = get(url)
        if result.status_code == 200:
            f.write(result.content)
            if sha1sum(output_file) == sha1:
                return True, "Downloaded and verified sha1"
            else:
                if retry:
                    return download_file(output_file, url, sha1, False)
                else:
                    return False, "Downloaded with bad sha1"
        else:
            if retry:
                return download_file(output_file, url, sha1, False)
            else:
                return False, f"{result.status_code}: {result.content}"


def get_checksum_headers(md5, sha1, sha256):
    return {"X-Checksum-Md5": md5, "X-Checksum-Sha1": sha1, "X-Checksum-Sha256": sha256}


def mirror_file(
    repo_work_dir,
    from_repo_uri,
    artifact_spec,
    to_repo_uri,
    to_repo_username,
    to_repo_password,
):
    state = {
        "download_url": f'{from_repo_uri}{artifact_spec["uri"]}',
    }
    # download
    d, f = artifact_spec["uri"].rsplit("/", maxsplit=1)
    output_dir = f"{repo_work_dir}{d}"
    makedirs(output_dir, exist_ok=True)
    state["download_output_file"] = f'{repo_work_dir}{artifact_spec["uri"]}'
    download_result, download_message = download_file(
        state["download_output_file"],
        state["download_url"],
        artifact_spec["sha1"],
        True,
    )
    # TODO Return checksums from download and use in upload
    state["download_result"] = download_result
    state["download_message"] = download_message

    # upload
    if download_result:
        content = read_file(state["download_output_file"])
        md5, sha1, sha256 = get_checksums(content)
        state["headers"] = {
            **get_checksum_headers(md5, sha1, sha256),
            "Content-Type": "application/octet-stream",
        }
        state["upload_url"] = f'{to_repo_uri}{artifact_spec["uri"]}'
        upload_result = put(
            state["upload_url"],
            data=content,
            headers=state["headers"],
            auth=(to_repo_username, to_repo_password),
        )
        state["upload_result"] = upload_result.status_code == 201
        state["upload_status_code"] = upload_result.status_code
        state["upload_response"] = upload_result.text
        state["result"] = state["upload_result"]

    return state


def not_folders(repo_files):
    return filter(lambda r: not r["folder"], repo_files)


def not_maven_metadata(repo_files):
    return filter(lambda r: not r["uri"].endswith("maven-metadata.xml"), repo_files)


def not_index(repo_files):
    return filter(lambda r: ".index" not in r["uri"], repo_files)


def latest_version(repo_files):
    version_map = {}
    for repo_file in repo_files:
        uri = repo_file["uri"]
        group_and_artifact_id = "/".join(uri.split("/")[:-2])
        version = uri.split("/")[-2]
        if group_and_artifact_id not in version_map:
            version_map[group_and_artifact_id] = []
        version_map[group_and_artifact_id].append(version)

    latest_version_map = {}
    for k, v in version_map.items():
        try:
            latest_version_map[k] = [
                get_latest_snapshot(v),
                get_latest_rc(v),
                get_latest_fixed(v),
            ]
        except ValueError:
            latest_version_map[k] = v

    latest_repo_files = []
    for repo_file in repo_files:
        uri = repo_file["uri"]
        group_and_artifact_id = "/".join(uri.split("/")[:-2])
        version = uri.split("/")[-2]
        if version in latest_version_map[group_and_artifact_id]:
            latest_repo_files.append(repo_file)

    return latest_repo_files


def filter_repo_files(repo_files, filters):
    print(f"all: {len(repo_files)}")
    for f in filters:
        repo_files = f(repo_files)
        repo_files = list(repo_files)
        print(f"{f.__name__}: {len(repo_files)}")
    return repo_files


# TODO Maybe use "Last downloaded" to filter out even more
# Note that files ending SNAPSHOT.* in from repo will get SNAPSHOT replaced
# by a timestamp in to repo and thus will not be found in to_uri
# on a rerun of this script. For now, nothing will be done about that.
def mirror(filters, max_processes=10):
    for repo_key in config.repositories:
        if type(repo_key) is str:
            from_repo_key = repo_key
            to_repo_key = repo_key
        else:
            from_repo_key, to_repo_key = repo_key

        from_repo_files = get_repo_content(
            from_repo_key, **config.from_artifactory_spec
        )["files"]
        filtered_from_repo_files = filter_repo_files(from_repo_files, filters)

        to_repo_files = get_repo_content(to_repo_key, **config.to_artifactory_spec)[
            "files"
        ]
        files_not_in_to_repo, files_not_in_from_repo = diff(
            filtered_from_repo_files, to_repo_files, only_keys=False
        )
        files_not_in_to_repo = list(files_not_in_to_repo)
        print("Not in to repo:", len(files_not_in_to_repo))

        from_repo_uri = f'{config.from_artifactory_spec["url"]}/{from_repo_key}'
        to_repo_uri = f'{config.to_artifactory_spec["url"]}/{to_repo_key}'
        yield from pmap_unordered(
            lambda artifact_spec: mirror_file(
                f"{config.work_dir}/{from_repo_key}",
                from_repo_uri,
                artifact_spec,
                to_repo_uri,
                config.to_artifactory_spec["username"],
                config.to_artifactory_spec["password"],
            ),
            files_not_in_to_repo,
            processes=max_processes,
        )


if __name__ == "__main__":
    verbose = True
    for n, m in zip(count(), mirror(config.filters)):
        if verbose or not m["result"]:
            print(n, dumps(m))
    print("Done!")



#!/usr/bin/env python3

import argparse
from fnmatch import fnmatch
from sys import argv

from vang.tfs.api import call
from vang.tfs.get_projects import get_projects


def list_release_definitions(organisations=None, projects=None, filter_name=None):
    if organisations:
        projects = get_projects(organisations, project_specs=True)
    if not projects:
        return []
    release_definitions = {
        release_definition["name"]: release_definition
        for project in projects
        for release_definition in call(f"/{project}/_apis/release/definitions")["value"]
    }
    if not filter_name:
        return release_definitions
    else:
        return {k: v for k, v in release_definitions.items() if fnmatch(k, filter_name)}


def main(
    organisations,
    projects,
    filter_name=None,
    ids=False,
    names=False,
    names_and_ids=False,
    urls=False,
    web_urls=False,
):
    for name, bd in list_release_definitions(
        organisations, projects, filter_name
    ).items():
        output = (name, bd)
        if ids:
            output = bd["id"]
        if names:
            output = bd["name"]
        if names_and_ids:
            output = name, bd["id"]
        if urls:
            output = bd["url"]
        if web_urls:
            output = bd["_links"]["web"]["href"]

        print(output)


def parse_args(args):
    parser = argparse.ArgumentParser(description="List TFS release definitions")
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        "-o", "--organisations", nargs="+", help="TFS organisations, e.g organisation"
    )
    required_group.add_argument(
        "-p", "--projects", nargs="+", help="TFS projects, e.g organisation/project"
    )

    parser.add_argument(
        "-fn",
        "--filter_name",
        help="Filters to definitions whose names equal this value. Append a * to filter to definitions whose names "
        "start with this value",
    )

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        "-i", "--ids", action="store_true", help="Get only release definition ids"
    )
    optional_group.add_argument(
        "-n", "--names", action="store_true", help="Get only release definition names"
    )
    optional_group.add_argument(
        "-ni",
        "--names_and_ids",
        action="store_true",
        help="Get only release definition names and ids (as tuples)",
    )
    optional_group.add_argument(
        "-u", "--urls", action="store_true", help="Get only release definition urls"
    )
    optional_group.add_argument(
        "-w",
        "--web_urls",
        action="store_true",
        help="Get only release definition web urls",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from collections import namedtuple
from os import environ
from pprint import pprint
from sys import argv

from requests import post

QueryArgs = namedtuple(
    "QueryArgs", "endpoint token query variables operation_name", defaults=({}, None)
)


def query_github(args: QueryArgs):
    json = {"query": args.query}
    if args.variables:
        json = json | {"variables": args.variables}
    if args.operation_name:
        json = json | {"operationName": args.operation_name}
    request = {
        "url": args.endpoint,
        "headers": {"Authorization": f"Bearer {args.token}"},
        "json": json,
    }
    r = post(**request)
    r.raise_for_status()
    return r.json()


def get_login(endpoint, token):
    query = "query { viewer { login }}"
    return query_github(QueryArgs(endpoint, token, query))["data"]["viewer"]["login"]


def get_repositories(endpoint, token, login, first=100, with_message=True):
    query = """query Repos($login: String!, $first: Int!, $withMessage: Boolean!) {
      repositoryOwner(login: $login) {
        repositories(first: $first) {
          nodes {
            name
            branches: refs(refPrefix: "refs/heads/", first: 100) {
              ...refFields
            }
            tags: refs(refPrefix: "refs/tags/", first: 100) {
              ...refFields
            }
          }
        }
      }
    }
    fragment refFields on RefConnection {
      nodes {
        name
        target {
          oid
          ... on Commit {
            message @include(if: $withMessage)
          }
        }
      }
    }"""
    variables = {"login": login, "first": first, "withMessage": with_message}
    response = query_github(
        QueryArgs(
            endpoint,
            token,
            query,
            variables,
        )
    )
    nodes = response["data"]["repositoryOwner"]["repositories"]["nodes"]

    def get_refs(refs):
        result = []
        for n in refs["nodes"]:
            r = {"name": n["name"], "commit": n["target"]["oid"]} | (
                {"message": n["target"]["message"]} if "message" in n["target"] else {}
            )
            result.append(r)
        return result

    return [
        {
            "name": r["name"],
            "branches": get_refs(r["branches"]),
            "tags": get_refs(r["tags"]),
        }
        for r in nodes
    ]


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(description="Query github")
    parser.add_argument(
        "--endpoint",
        "-e",
        default="https://api.github.com/graphql",
        help="The Github GraphQL endpoint",
    )
    parser.add_argument(
        "--token",
        "-t",
        default=environ.get("GITHUB_TOKEN", ""),
        help="The Github authorisation token",
    )
    parser.add_argument(
        "--login",
        "-l",
        default=None,
        help="The Github login",
    )
    parser.add_argument(
        "--first",
        "-f",
        default=100,
        help="The number of Github repos",
    )
    parser.add_argument(
        "--with_message",
        "-w",
        default=True,
        help="Include commit messages",
    )
    return parser.parse_args(args)


def main(
    endpoint: str, token: str, login: str, first: int, with_message: bool
) -> None:  # pragma: no cover
    login = login or get_login(endpoint, token)
    repositories = get_repositories(endpoint, token, login, first, with_message)
    for r in repositories:
        pprint(r)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import itertools
from sys import argv

from vang.bitbucket.api import get_all
from vang.core.core import pmap_unordered


def get_repos(project, only_name=False, only_spec=False):
    for value in get_all(f"/rest/api/1.0/projects/{project}/repos"):
        if only_name:
            yield value["slug"]
        elif only_spec:
            yield value["project"]["key"], value["slug"]
        else:
            yield value


def get_all_repos(projects, max_processes=10, only_name=False, only_spec=False):
    return itertools.chain.from_iterable(
        pmap_unordered(
            lambda p: get_repos(p, only_name, only_spec),
            projects,
            processes=max_processes,
        )
    )


def main(projects, name, repo_specs):
    for repo in get_all_repos(projects, only_name=name, only_spec=repo_specs):
        if repo_specs:
            print(f"{repo[0]}/{repo[1]}")
        else:
            print(repo)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Get repos from Bitbucket")
    parser.add_argument("projects", nargs="+", help="Project keys")
    group = parser.add_mutually_exclusive_group()
    group.add_argument("-n", "--name", help="Print only repo name", action="store_true")
    group.add_argument(
        "-r", "--repo_specs", help="Print only project_key/name", action="store_true"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from vang.jenkins.copy_job import copy_job


def copy_jobs(jobs, from_jenkins_spec, to_jenkins_spec, replacements=[], verbose=False):
    for job in jobs:
        from_name = None
        to_name = None
        job_replacements = replacements
        try:
            if type(job) is str:
                to_name = from_name = job
            elif type(jobs) is list and len(job) == 2:
                from_name, to_name = job
            elif type(jobs) is list and len(job) == 3:
                from_name, to_name, job_replacements = job

            if (
                type(from_name) != str
                or type(to_name) != str
                or not (type(job_replacements) is list or callable(job_replacements))
            ):
                raise ValueError(
                    "job must be a string, a pair of strings or a pair of string and a callable"
                )

            response = copy_job(
                from_name, to_name, from_jenkins_spec, to_jenkins_spec, job_replacements
            )
            response_code = response["response_code"]
            print(
                "Created copy" if response_code == 200 else "Failed to create copy",
                to_name,
                response_code,
            )
            if verbose:
                print(response)
        except IOError as r:
            print("IOError while trying to create copy", to_name, r)
        except ValueError as v:
            print("ValueError while trying to create copy", to_name, v)



#!/usr/bin/env python3
import argparse
import logging
from json import loads
from os import environ
from os.path import basename
from pprint import pprint
from sys import argv
from typing import Dict

from requests import get

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = "https://dev.azure.com"


def list_repos(
    token: str,
    organization: str,
    project: str,
    verify_certificate: bool = True,
    include_links: bool = False,
    include_all_urls: bool = False,
    include_hidden: bool = False,
    api_version: str = "6.1-preview.1",
) -> Dict:
    url = f"{base_url}/{organization}/{project}/_apis/git/repositories?includeLinks={include_links}&includeAllUrls={include_all_urls}&includeHidden={include_hidden}&api-version={api_version}"
    params = {"url": url, "auth": ("", token), "verify": verify_certificate}
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = get(**params)
    logger.info(f"response.status_code: {response.status_code}")
    logger.info(f"response.text: {response.text}")
    response.raise_for_status()
    return loads(response.text)


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(description="List repos")
    parser.add_argument(
        "--token",
        default=environ.get("AZDO_TOKEN", ""),
        help="The Azure DevOps authorisation token",
    )
    parser.add_argument(
        "--organisation",
        default=environ.get("AZDO_ORGANISATION", ""),
        help="The Azure DevOps organisation",
    )
    parser.add_argument(
        "--project",
        default=environ.get("AZDO_PROJECT", ""),
        help="The Azure DevOps project",
    )
    parser.add_argument(
        "-au",
        "--azure_devops_url",
        default="https://dev.azure.com",
        help="The Azure DevOps REST API base url",
    )

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        "-n", "--names", action="store_true", help="Get only repo names"
    )
    optional_group.add_argument(
        "-r",
        "--repo_specs",
        help="Print only organisation/project/name",
        action="store_true",
    )
    optional_group.add_argument(
        "-u", "--urls", action="store_true", help="Get only repo urls"
    )

    return parser.parse_args(args)


def main(
    token: str,
    organisation: str,
    project: str,
    azure_devops_url: str,
    names: bool,
    repo_specs: bool,
    urls: bool,
) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    repos = list_repos(token, organisation, project)
    if not any((names, repo_specs, urls)):
        pprint(repos)
    else:
        for r in repos["value"]:
            if names:
                print(r["name"])
            elif repo_specs:
                print(f'{organisation}/{project}/{r["name"]}')
            elif urls:
                print(r["remoteUrl"])


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from os import environ

from requests import delete, get, post, put


def call(
    uri,
    request_data=None,
    method="GET",
    only_response_code=False,
    rest_url=environ.get("JENKINS_REST_URL", None),
    username=environ.get("JENKINS_USERNAME", None),
    password=environ.get("JENKINS_PASSWORD", None),
    verify_certificate=not environ.get("JENKINS_IGNORE_CERTIFICATE", None),
):
    """Makes a REST call to Jenkins rest api.
    May use three environment variables:
    * JENKINS_REST_URL, e.g. http://myorg.com/stash
    * JENKINS_USERNAME, the jenkins username
    * JENKINS_PASSWORD, the jenkins password

    Args:
        uri (str): e.g. "/api/json"
        request_data (dict): the JSON request
        method: http method
        only_response_code: default False
        rest_url: default environ.get('JENKINS_REST_URL', None)
        username: default environ.get('JENKINS_USERNAME', None)
        password: default environ.get('JENKINS_PASSWORD', None)
        verify_certificate: True if https certificate should be verified

    Return:
          the response
    """
    return call_url(
        f"{rest_url}{uri}",
        request_data,
        method,
        only_response_code,
        username,
        password,
        verify_certificate,
    )


def call_url(
    url,
    request_data=None,
    method="GET",
    only_response_code=False,
    username=environ.get("JENKINS_USERNAME", None),
    password=environ.get("JENKINS_PASSWORD", None),
    verify_certificate=True,
):
    m = {
        "DELETE": delete,
        "GET": get,
        "POST": post,
        "PUT": put,
    }[method]

    params = {"url": url, "auth": (username, password), "verify": verify_certificate}
    if request_data:
        params["json"] = request_data

    response = m(**params)
    return (
        response.status_code
        if only_response_code
        else response.json()
        if response.text
        else response.status_code()
    )



#!/usr/bin/env python3
""" Maven pom utilities. """

from glob import glob
from os.path import realpath, normpath
from xml.etree.ElementTree import parse

NAME_SPACES = {"ns": "http://maven.apache.org/POM/4.0.0"}


def get_pom_paths(root):
    """Returns the absolute path of poms that recursively are found in root"""
    return [realpath(p) for p in glob(f"{root}/**/pom.xml", recursive=True)]


def get_pom_info(pom_path):
    """Returns a dictionary with pom_path, artifact_id,
    group_id, version and packaging."""
    e = parse(normpath(pom_path))
    info = {
        "pom_path": pom_path,
        "artifact_id": e.findtext("ns:artifactId", namespaces=NAME_SPACES)
        or e.findtext("artifactId"),
        "group_id": e.findtext("ns:groupId", namespaces=NAME_SPACES)
        or e.findtext("groupId"),
        "version": e.findtext("ns:version", namespaces=NAME_SPACES)
        or e.findtext("version"),
        "packaging": e.findtext("ns:packaging", namespaces=NAME_SPACES)
        or e.findtext("packaging")
        or "jar",
    }
    if not info["group_id"]:
        info["group_id"] = e.findtext(
            "ns:parent/ns:groupId", namespaces=NAME_SPACES
        ) or e.findtext("parent/groupId")
    if not info["version"]:
        info["version"] = e.findtext(
            "ns:parent/ns:version", namespaces=NAME_SPACES
        ) or e.findtext("parent/version")

    return info



#!/usr/bin/env python3
import argparse
from multiprocessing.dummy import Pool
from sys import argv

from vang.bitbucket.api import call
from vang.bitbucket.utils import get_repo_specs


def get_repo_default_branch(spec):
    return spec, call(
        f"/rest/api/1.0/projects/{spec[0]}/repos/{spec[1]}/branches/default"
    )


def get_default_branch(repo_specs, max_processes=10):
    with Pool(processes=max_processes) as pool:
        return pool.map(get_repo_default_branch, repo_specs)


def main(dirs, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, response in get_default_branch(specs):
        print(f'{spec[0]}/{spec[1]}: {response["displayId"]}')


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Get default repository branches from Bitbucket"
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Git directories to extract repo information from",
    )
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from functools import reduce


def chunks(l, n):
    """Yield successive n-sized chunks from l."""
    for i in range(0, len(l), n):
        yield l[i : i + n]


def thread_first(arg, *partials):
    """
    Works like Clojure thread first.
    Example 1:
    from operator import add
    thread_first('a',
                [add, 'b'],
                [lambda x: x],
                [add, 'c'])
    > 'abc'

    Example 2:
    from operator import add
    thread_first('abc',
                (str.replace, 'b', 'x'),
                (add, 'y'))
    > 'axcy'
    """
    return reduce(
        lambda mem, p: p[0](mem, *p[1:]) if len(p) > 1 else p[0](mem), partials, arg
    )


def thread_last(arg, *partials):
    """
    Works like Clojure thread last.
    Example:
    from operator import add
    thread_last('a',
                [add, 'b'],
                [lambda x: x],
                [add, 'c'])
    > 'cba'
    """
    return reduce(
        lambda mem, p: p[0](*p[1:], mem) if len(p) > 1 else p[0](mem), partials, arg
    )



#!/usr/bin/env python3
import argparse
from os import remove, rename, replace, walk
from os.path import join, sep
from re import fullmatch, sub
from sys import argv
from typing import Callable, Iterable


def get_replace_function(regexp=False) -> Callable[[str, str, str], str]:
    return lambda s, old, new: sub(old, new, s) if regexp else s.replace(old, new)


def _replace_in_file(
    old: str, new: str, path: str, replace_function: Callable[[str, str, str], str]
) -> None:
    content_changed = False
    with open(path, "tr", encoding="UTF-8", errors="ignore",) as old_file, open(
        path + ".tmp",
        "tw",
        encoding="UTF-8",
    ) as new_file:
        for line in old_file:
            new_line = replace_function(line, old, new)
            content_changed = content_changed or new_line != line
            new_file.write(new_line)
    replace(path + ".tmp", path) if content_changed else remove(path + ".tmp")


def _replace_file(
    old: str,
    new: str,
    path: str,
    file: str,
    replace_function: Callable[[str, str, str], str],
) -> None:
    new_file = replace_function(file, old, new)
    if new_file != file:
        rename(join(path, file), join(path, new_file))


def _in(name: str, regexps: Iterable[str]) -> bool:
    return any(fullmatch(name, e) for e in regexps)


def _rsr(
    root: str,
    excludes: Iterable[str],
    old: str,
    new: str,
    replace_function: Callable[[str, str, str], str],
) -> None:
    for dir_path, dir_names, files in walk(root, False):
        if not any(_in(d, excludes) for d in dir_path.split(sep)):
            for file in files:
                if not _in(file, excludes):
                    _replace_in_file(old, new, join(dir_path, file), replace_function)
                    _replace_file(old, new, dir_path, file, replace_function)

            for dir_name in dir_names:
                if not _in(dir_name, excludes):
                    _replace_file(old, new, dir_path, dir_name, replace_function)


def rsr(
    old: str,
    new: str,
    dirs: Iterable[str],
    replace_function: Callable[[str, str, str], str],
) -> None:
    for d in dirs:
        _rsr(d, [".git", ".gitignore", "target"], old, new, replace_function)


def main(old: str, new: str, dirs: Iterable[str], regexp: bool) -> None:
    rsr(old, new, dirs, get_replace_function(regexp))


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Recursive search and replace of "
        "directories, files and file contents"
    )
    parser.add_argument("old", help="Old value")
    parser.add_argument("new", help="New value")
    parser.add_argument("-d", "--dirs", nargs="*", default=["."])
    parser.add_argument("-r", "--regexp", action="store_true")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import logging
from json import loads
from os import environ
from os.path import basename
from sys import argv
from typing import Iterable

from requests import get

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = "https://dev.azure.com"


def list_projects(
    token: str,
    organisation: str,
    names: bool,
    project_specs: bool,
    verify_certificate: bool = True,
    api_version: str = "6.1-preview.4",
) -> Iterable:
    url = f"{base_url}/{organisation}/_apis/projects?api-version={api_version}"
    params = {"url": url, "auth": ("", token), "verify": verify_certificate}
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = get(**params)
    logger.info(f"response.status_code: {response.status_code}")
    logger.info(f"response.text: {response.text}")
    response.raise_for_status()

    projects = loads(response.text)
    if not any((names, project_specs)):
        return projects["value"]
    return [
        r["name"] if names else f'{organisation}/{r["name"]}' for r in projects["value"]
    ]


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(description="List projects")
    parser.add_argument(
        "--token",
        default=environ.get("AZDO_TOKEN", ""),
        help="The Azure DevOps authorisation token",
    )
    parser.add_argument(
        "--organisation",
        default=environ.get("AZDO_ORGANISATION", ""),
        help="The Azure DevOps organisation",
    )
    parser.add_argument(
        "-au",
        "--azure_devops_url",
        default="https://dev.azure.com",
        help="The Azure DevOps REST API base url",
    )

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        "-n", "--names", action="store_true", help="Get only project names"
    )
    optional_group.add_argument(
        "-r",
        "--project_specs",
        help="Print only organisation/project",
        action="store_true",
    )

    return parser.parse_args(args)


def main(
    token: str,
    organisation: str,
    azure_devops_url: str,
    names: bool,
    project_specs: bool,
) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    for p in list_projects(token, organisation, names, project_specs):
        print(p)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from os import name, system

from vang.maven.pom import get_pom_info


def get_artifact_id(pom_path):
    return get_pom_info(pom_path)["artifact_id"]


def main():
    artifact_id = get_artifact_id("./pom.xml")
    if name == "posix":
        system(f'echo "{artifact_id}\c" | pbcopy')
        print(f'"{artifact_id}" copied to clipboard')
    else:
        print(artifact_id)


if __name__ == "__main__":  # pragma: no cover
    main()



#!/usr/bin/env python3
import argparse
import itertools
from multiprocessing.dummy import Pool
from sys import argv

from vang.bitbucket.api import get_all
from vang.bitbucket.utils import get_repo_specs


# def get_tags_page(spec, tag, limit, start, order_by='MODIFICATION'):
#     response = call(
#         f'/rest/api/1.0/projects/{spec[0]}/repos/{spec[1]}/tags?'
#         f'filterText={tag}&limit={limit}&start={start}&orderBy={order_by}')
#     return response['size'], response['values'], response[
#         'isLastPage'], response.get('nextPageStart', -1)


def get_all_tags(spec, tag="", order_by="MODIFICATION"):
    for t in get_all(
        f"/rest/api/1.0/projects/{spec[0]}/repos/{spec[1]}/tags",
        {"filterText": tag, "orderBy": order_by} if tag else {"orderBy": order_by},
    ):
        yield spec, t
    # limit = 25
    # start = 0
    # is_last_page = False
    #
    # while not is_last_page:
    #     size, values, is_last_page, start = get_tags_page(
    #         spec, tag, limit, start)
    #
    #     if size:
    #         for value in values:
    #             yield spec, value


def get_tags(specs, tag="", max_processes=10):
    with Pool(processes=max_processes) as pool:
        return itertools.chain.from_iterable(
            pool.map(lambda s: get_all_tags(s, tag), specs)
        )


def main(tag="", name=False, dirs=None, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, response in get_tags(specs, tag):
        if name:
            print(response["displayId"])
        else:
            print(f'{spec[0]}/{spec[1]}: {response["displayId"]}')


def parse_args(args):
    parser = argparse.ArgumentParser(description="Get repository tags from Bitbucket")
    parser.add_argument("-t", "--tag", help="Tag filter", default="")
    parser.add_argument("-n", "--name", help="Print only tag name", action="store_true")
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Git directories to extract repo information from",
    )
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.bitbucket.create_repo import create_repo
from vang.bitbucket.enable_webhooks import enable_repo_web_hook
from vang.bitbucket.get_clone_urls import get_clone_urls
from vang.bitbucket.set_default_branches import set_repo_default_branch
from vang.misc.s import get_zipped_cases
from vang.pio.rsr import get_replace_function, rsr
from vang.pio.shell import run_command


def setup(project, repo, src_branch, dest_project, dest_repo, dest_branch, work_dir):
    clone_url = [c for _, p, r, c in get_clone_urls([project]) if r == repo][0]
    dest_repo_dir = f"{work_dir}/{dest_project}/{dest_repo}"
    run_command(f"git clone -b {src_branch} {clone_url} {dest_repo_dir}")
    run_command("rm -rf .git", return_output=True, cwd=dest_repo_dir)
    run_command("git init", return_output=True, cwd=dest_repo_dir)
    if not dest_branch == "master":
        run_command(
            f"git checkout -b {dest_branch}", return_output=True, cwd=dest_repo_dir
        )
    return clone_url, dest_repo_dir


def commit_all(repo_dir):
    run_command("git add --all", return_output=True, cwd=repo_dir)
    run_command(
        'git commit -m "Initial commit"',
        return_output=True,
        cwd=repo_dir,
    )


def update(repo, dest_repo, dest_repo_dir):
    for old, new in get_zipped_cases([repo, dest_repo]):
        rsr(old, new, [dest_repo_dir], get_replace_function(False))


def create_and_push_to_dest_repo(
    dest_project, dest_repo, dest_branch, dest_repo_dir, webhook=None
):
    dest_repo_origin = create_repo(dest_project, dest_repo)["links"]["clone"][0]["href"]
    if webhook:
        enable_repo_web_hook([dest_project, dest_repo], webhook)
    run_command(
        f"git remote add origin {dest_repo_origin}",
        return_output=True,
        cwd=dest_repo_dir,
    )
    run_command(
        f"git push -u origin {dest_branch}", return_output=True, cwd=dest_repo_dir
    )
    set_repo_default_branch((dest_project, dest_repo), dest_branch)
    return dest_repo_origin


def parse_args(args):
    parser = ArgumentParser(
        description="Create a new repo based on template repo.\nExample: "
        "create_from_template PCS1806 foo PCS1806 bar -sb "
        "develop -db 1.0.x -d . -w http://my-host:7777/wildcat/webhook/"
    )
    parser.add_argument(
        "src_project", help="The project from which to create the new repo (must exist)"
    )
    parser.add_argument(
        "src_repo", help="The repo from which to create the new repo (must exist)"
    )
    parser.add_argument(
        "dest_project", help="The project in which to create the new repo (must exist)"
    )
    parser.add_argument("dest_repo", help="The new repo (must not exist)")
    parser.add_argument(
        "-sb", "--src_branch", default="develop", help="The branch to use, e.g. develop"
    )
    parser.add_argument(
        "-db",
        "--dest_branch",
        help="The branch to create, e.g. develop. "
        "Will be set to src_branch if not specified. "
        "It will be set as default branch on created repo",
    )
    parser.add_argument(
        "-d",
        "--work_dir",
        default=".",
        help="The directory to create local repo in (directory work_dir/src_project/src_repo must not exists)",
    )
    parser.add_argument(
        "-w", "--webhook", default=False, help="Webhook url to add to the new repo"
    )
    return parser.parse_args(args)


def main(
    src_project,
    src_repo,
    src_branch,
    dest_project,
    dest_repo,
    dest_branch,
    work_dir,
    webhook,
):
    if not dest_branch:
        dest_branch = src_branch
    clone_url, dest_repo_dir = setup(
        src_project,
        src_repo,
        src_branch,
        dest_project,
        dest_repo,
        dest_branch,
        work_dir,
    )
    update(src_repo, dest_repo, dest_repo_dir)
    commit_all(dest_repo_dir)
    dest_repo_origin = create_and_push_to_dest_repo(
        dest_project, dest_repo, dest_branch, dest_repo_dir, webhook
    )
    print("Created", dest_repo_origin)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
import re
from json import dumps
from sys import argv

from vang.tfs.api import call_url
from vang.tfs.list_build_definitions import list_build_definitions


def queue_build_definitions(
    organisations=None,
    projects=None,
    filter_name=None,
):
    name_and_urls = [
        (name, bd["id"], bd["url"])
        for name, bd in list_build_definitions(
            organisations, projects, filter_name
        ).items()
    ]
    for name, definition_id, url in name_and_urls:
        build_url = re.sub(r"_apis/.*", "", url) + "_apis/build/builds?api-version=3.2"
        yield name, call_url(
            build_url, {"definition": {"id": definition_id}}, method="POST"
        )


def main(organisations, projects, filter_name=None, format_output=False):
    for name, result in queue_build_definitions(organisations, projects, filter_name):
        json = dumps(result, indent=4) if format_output else dumps(result)
        print(json)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Queue TFS build definitions")
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        "-o", "--organisations", nargs="+", help="TFS organisations, e.g organisation"
    )
    required_group.add_argument(
        "-p", "--projects", nargs="+", help="TFS projects, e.g organisation/project"
    )

    parser.add_argument(
        "-fn",
        "--filter_name",
        help="Filters to definitions whose names equal this value. Append a * to filter to definitions whose names "
        "start with this value",
    )

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        "-f", "--format_output", action="store_true", help="Format output"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import environ
from sys import argv

from requests import post

from vang.jenkins.download_configs import get_config


def get_default_jenkins_spec():
    return {
        "url": environ.get("JENKINS_REST_URL", None),
        "username": environ.get("JENKINS_USERNAME", None),
        "password": environ.get("JENKINS_PASSWORD", None),
        "verify_certificate": not environ.get("JENKINS_IGNORE_CERTIFICATE", None),
    }


def create_params(jenkins_spec):
    return {
        "auth": (jenkins_spec["username"], jenkins_spec["password"]),
        "verify": jenkins_spec["verify_certificate"],
    }


def create_job(jenkins_spec, name, config):
    params = create_params(jenkins_spec)
    params["url"] = f'{jenkins_spec["url"]}/createItem?name={name}'
    params["data"] = config
    params["headers"] = {"Content-Type": "application/xml"}
    response = post(**params)
    return response.status_code


def update_config(config, replacements):
    if callable(replacements):
        return replacements(config)
    else:
        for r in replacements:
            config = config.replace(*r)
        return config


def copy_job(
    from_name,
    to_name,
    from_jenkins_spec=get_default_jenkins_spec(),
    to_jenkins_spec=get_default_jenkins_spec(),
    replacements=None,
):
    from_response_code, from_config = get_config(from_name, from_jenkins_spec)
    to_config = update_config(from_config, replacements)
    to_response_code = create_job(to_jenkins_spec, to_name, to_config)
    return {
        "from_name": from_name,
        "to_name": to_name,
        "from_config": from_config,
        "to_config": to_config,
        "response_code": to_response_code,
    }


def parse_args(args):
    parser = argparse.ArgumentParser(description="Copy Jenkins job")
    parser.add_argument("from_name", help="Name of job to copy")
    parser.add_argument("to_name", help="Name of job copy")
    parser.add_argument(
        "-r",
        "--replacements",
        nargs="*",
        default=[],
        help="Pair of replacements, e.g. old1 new1 old2 new2",
    )
    return parser.parse_args(args)


def main(from_name, to_name, replacements=()):
    response = copy_job(
        from_name, to_name, replacements=zip(replacements[0::2], replacements[1::2])
    )
    response_code = response["response_code"]
    print(
        "Created" if response_code == 200 else "Failed to create",
        to_name,
        response_code,
    )


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.tfs.api import call_url
from vang.tfs.definition_utils import get_definition, get_definition_name
from vang.tfs.list_release_definitions import list_release_definitions


def get_current_definition(organisation, project, definition_name):
    return list_release_definitions(
        projects=(f"{organisation}/{project}",), filter_name=definition_name
    )[definition_name]


def get_release_definition(
    template,
    organisation,
    project,
    repo,
    branch,
    definition_name,
    definition_id,
    revision,
    comment=None,
):
    additions = {"id": definition_id, "revision": revision}
    if comment:
        additions["comment"] = comment
    return get_definition(
        template,
        {
            "name": definition_name,
            "organisation": organisation,
            "project": project,
            "repo": repo,
            "branch": branch,
        },
        additions,
    )


def update_release_definition(definition_url, release_definition):
    return call_url(
        f"{definition_url}?api-version=3.2-preview",
        request_data=release_definition,
        method="PUT",
        only_response_code=True,
    )


def main(project, repo, branch, template, comment=None):
    organisation, project = project.split("/")
    definition_name = get_definition_name(project, repo, branch)
    definition = get_current_definition(organisation, project, definition_name)
    definition_url = definition["url"]
    definition_id = definition["id"]
    revision = definition["revision"]
    response = update_release_definition(
        definition_url,
        get_release_definition(
            template,
            organisation,
            project,
            repo,
            branch,
            definition_name,
            definition_id,
            revision,
            comment,
        ),
    )
    print(response)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Update TFS release definitions")
    parser.add_argument("project", help="TFS projects, e.g organisation/project")
    parser.add_argument("repo", help="The TFS git repo name, e.g. spam.eggs")
    parser.add_argument("branch", help="The TFS git repo branch, e.g. 1.0.x")
    parser.add_argument(
        "template",
        help="The TFS release definition template file, e.g. release_definition_template.json",
    )
    parser.add_argument(
        "-c",
        "--comment",
        help="The comment to add to the release definition, e.g. a commit sha of the release definition template",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import sys
from itertools import chain

from vang.bitbucket.api import get_all
from vang.bitbucket.utils import get_repo_specs
from vang.core.core import pmap_unordered


# This is not used in this script but still a useful function :)
def get_branch_commits(
    repo_specs_and_branches, take=sys.maxsize, max_processes=10, filter_function=None
):
    def inner(rsb):
        spec = rsb[0]
        branches = rsb[1]
        result = {}
        for b in branches:
            commits = get_all(
                f"/rest/api/1.0/projects/{spec[0]}/repos/{spec[1]}/commits",
                {"until": b},
                take,
            )
            result[b] = filter_function(commits) if filter_function else commits

        return result

    return pmap_unordered(
        lambda r: {r[0]: inner(r)}, repo_specs_and_branches, processes=max_processes
    )


def get_commits(repo_specs, branches=tuple([""]), take=sys.maxsize, max_processes=10):
    return pmap_unordered(
        lambda s: list(
            [
                s,
                chain(
                    [
                        (
                            b,
                            get_all(
                                f"/rest/api/1.0/projects/{s[0]}/repos/{s[1]}/commits",
                                {"until": b},
                                take,
                            ),
                        )
                        for b in branches
                    ]
                ),
            ]
        ),
        repo_specs,
        processes=max_processes,
    )


def main(take, branches, sha=False, dirs=None, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, commits in get_commits(specs, branches, take=take):
        if sha:
            for b, cs in commits:
                for c in cs:
                    # print(b, c['id'])
                    print(c["id"])
        else:
            print(f"{spec[0]}/{spec[1]}: {list(commits)}")


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Get repository commits from Bitbucket"
    )
    parser.add_argument(
        "-b", "--branches", nargs="*", help="Branches filter", default=("",)
    )
    parser.add_argument(
        "--take",
        "-t",
        type=int,
        help="The number of commit from and including HEAD. -1 for all commits",
        default=sys.maxsize,
    )
    parser.add_argument("-s", "--sha", help="Print only sha", action="store_true")
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Git directories to extract commit information from",
    )
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(sys.argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import sys
from functools import partial
from multiprocessing.dummy import Pool
from traceback import print_exc

import vang.bitbucket.clone_repos as clone_repos
from vang.bitbucket.create_repo import create_repo
from vang.bitbucket.set_default_branches import set_repo_default_branch
from vang.pio.command_all import get_work_dirs
from vang.pio.shell import run_command


def fork_repo(fork_project, branches, git_dir):
    try:
        for b in reversed(branches):
            run_command(f"git checkout {b}", return_output=True, cwd=git_dir)
        rc, original_origin = run_command(
            "git remote get-url origin", return_output=True, cwd=git_dir
        )
        repo = original_origin.split("/")[-1][:-4]
        new_origin = (
            "/".join(original_origin.split("/")[:-2])
            + "/"
            + fork_project.lower()
            + "/"
            + repo
            + ".git"
        )
        create_repo(fork_project, repo)
        run_command(
            f"git remote set-url origin {new_origin}", return_output=True, cwd=git_dir
        )
        run_command("git remote prune origin", return_output=True, cwd=git_dir)
        for b in branches:
            run_command(f"git push -u origin {b}", return_output=True, cwd=git_dir)
        set_repo_default_branch((fork_project, repo), branches[0])

        print(f"{fork_project}/{repo}: {new_origin}")
        return (fork_project, repo), new_origin
    except OSError:  # pragma: no cover
        print(f"Error forking {git_dir}")
        print_exc(file=sys.stdout)


def fork_repos(
    fork_project, branches, work_dir=".", repos=None, projects=None, max_processes=10
):
    clone_repos.main(work_dir, projects, repos, branch=branches[0], flat=True)
    git_dirs = get_work_dirs(".git/", work_dir)
    with Pool(processes=max_processes) as pool:
        pool.map(partial(fork_repo, fork_project, branches), git_dirs)


def main(fork_project, branches, work_dir=".", repos=None, projects=None):
    fork_repos(fork_project, branches, work_dir, repos, projects)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="""
       Fork Bitbucket repos by cloning repos,
       create new repos in fork project,
       set them as origin
       and push branches to new repos
    """
    )
    parser.add_argument("fork_project", help="Fork project (must exist)")
    parser.add_argument(
        "-b",
        "--branches",
        nargs="*",
        help="The branches to fork, e.g. develop master. "
        "The first will be set as default branch",
        default=["develop"],
    )
    parser.add_argument(
        "-d",
        "--work_dir",
        default=".",
        help="The directory to clone into. Preferably an empty directory!",
    )
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(sys.argv[1:]).__dict__)



#!/usr/bin/env python3

from vang.bitbucket.get_repos import get_all_repos
from vang.pio.shell import run_command


def get_branch(git_dir):
    return run_command("git rev-parse --abbrev-ref HEAD", True, git_dir)[1]


def get_clone_url(git_dir):
    return_code, url = run_command(
        f"git --git-dir {git_dir} remote get-url origin",
        True,
        git_dir,
        False,
    )
    if return_code:
        return run_command(
            f"git --git-dir {git_dir}/.git remote get-url origin",
            True,
            git_dir,
        )[1]
    else:
        return url


def get_project_and_repo(git_dir):
    project, repo = get_clone_url(git_dir)[:-4].split("/")[-2:]
    return project.upper(), repo


def get_repo_specs(dirs=None, repos=None, projects=None):
    if projects:
        return get_all_repos(projects, only_spec=True, max_processes=10)
    elif repos:
        return (tuple(repo.split("/")) for repo in repos)
    else:
        return (get_project_and_repo(a_dir) for a_dir in dirs)



#!/usr/bin/env python3
from argparse import ArgumentParser
from re import compile
from sys import argv


def get_split(s):
    for ch in ("_", "-"):
        if ch in s:
            return s.split(ch)
    return compile("[A-Z]?[^A-Z]+").findall(s)


def get_cases(s):
    split = get_split(s)
    capital = [w.capitalize() for w in split]
    lower = [w.lower() for w in split]
    upper = [w.upper() for w in split]
    return [
        "".join([lower[0]] + capital[1:]),
        "".join(capital),
        "".join(lower),
        "".join(upper),
        "_".join(lower),
        "_".join(upper),
        "-".join(lower),
        "-".join(upper),
    ]


def get_zipped_cases(strings):
    return zip(*[get_cases(s) for s in strings])


def parse_args(args):
    parser = ArgumentParser(
        description="Prints various string representation of provided strings"
    )
    parser.add_argument("strings", nargs="+")
    return parser.parse_args(args)


def main(strings):
    for items in get_zipped_cases(strings):
        print(" ".join(items))


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import sys
import termios
import threading
from argparse import ArgumentParser
from glob import glob
from itertools import chain, count
from os import system
from subprocess import PIPE, Popen, STDOUT
from time import sleep

lock = threading.Lock()
action = None
play_process = None


def set_action(k, e):
    global action
    action = k
    e.set()


def stop():
    with lock:
        global play_process
        if play_process and play_process.poll() is None:
            try:
                play_process.kill()
            except SystemError:
                pass


def process_monitor(e):
    global play_process
    while True:
        with lock:
            if play_process and play_process.poll() is not None:
                set_action("n", e)
        sleep(1)


def play(track):
    global play_process
    stop()
    print("Playing", track)
    with lock:
        play_process = Popen(["afplay", track], stdout=PIPE, stderr=STDOUT)


def get_action(e):
    while True:
        fd = sys.stdin.fileno()

        oldterm = termios.tcgetattr(fd)
        newattr = termios.tcgetattr(fd)
        newattr[3] = newattr[3] & ~termios.ICANON & ~termios.ECHO
        termios.tcsetattr(fd, termios.TCSANOW, newattr)

        try:
            c = sys.stdin.read(1)
            set_action(str(c), e)
        except IOError:
            pass

        termios.tcsetattr(fd, termios.TCSAFLUSH, oldterm)


def play_all(tracks):
    global action
    tracks = sorted(tracks)
    keyed_tracks = list(
        zip(chain(range(97, 122), range(65, 90)), [s.split("/")[-1] for s in tracks])
    )

    e = threading.Event()
    threading.Thread(target=get_action, args=(e,), daemon=True).start()
    index = 0
    threading.Thread(target=process_monitor, args=(e,), daemon=True).start()
    play(tracks[index])
    while action != "s":
        e.wait()
        e.clear()
        if action == "b":
            play(tracks[index])
        if action == "c":
            print("Currently playing", tracks[index])
        if action == "n":
            index = (index + 1) % len(tracks)
            play(tracks[index])
        if action == "p":
            index = index - 1 if index > 0 else len(tracks) - 1
            play(tracks[index])
        if action == "s":
            stop()
        if action == "l":
            system("clear")
            for i, p in keyed_tracks:
                print(chr(i), p)
            e.wait()
            e.clear()
            if action != chr(27):  # Escape character
                for i, entry in zip(count(0), keyed_tracks):
                    if chr(entry[0]) == action:
                        index = i
                        play(tracks[index])
            action = ""

    print("Done!")


def get_audio_files(path):
    return glob(f"{path}/**/*.wav", recursive=True)


def parse_args(args):
    parser = ArgumentParser(description="Play wav files on Mac")
    parser.add_argument(
        "-d", "--wav_dir", help="Directory containing Wav-files", default="."
    )
    return parser.parse_args(args)


def main(wav_dir):
    play_all(get_audio_files(wav_dir))


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(sys.argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import name, system
from sys import argv

from vang.bitbucket.api import call


def create_repo(project, repo):
    uri = f"/rest/api/1.0/projects/{project}/repos"
    # request_data = f'{{"name":"{repo}","scmId":"git","forkable":true}}'
    request_data = {"name": repo, "scmId": "git", "forkable": True}
    return call(uri, request_data, "POST")


def main(project, repository):
    response = create_repo(project, repository)
    commands = (
        "    git remote add origin "
        f'{response["links"]["clone"][0]["href"]}\n'
        "    git push -u origin develop"
    )
    print(
        "If you already have code ready to be pushed to this repository "
        "then run this in your terminal."
    )
    print(commands)
    if name == "posix":
        system(f'echo "{commands}\\c" | pbcopy')
        print("(The commands are copied to the clipboard)")


def parse_args(args):
    parser = argparse.ArgumentParser(description="Create Bitbucket repository")
    parser.add_argument("project", help="Project key")
    parser.add_argument("repository", help="Repository")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import logging
from datetime import datetime
from re import fullmatch

from vang.bitbucket.api import get_all
from vang.bitbucket.get_projects import get_projects
from vang.bitbucket.get_repos import get_repos
from vang.core.core import pmap_unordered, get_in
from vang.jenkins.get_jobs import get_jobs

logging.basicConfig(level=logging.INFO)


def matches(v, pattern):
    return fullmatch(pattern, v)


def young_enough(commit_datetime, max_head_commit_age):
    return (datetime.now() - commit_datetime).days < max_head_commit_age


def complement_and_filter_repos(repos, branch_pattern, max_head_commit_age):
    def inner(r):
        branches = {}
        for b in get_all(
            f'/rest/api/1.0/projects/{r["key"]}/repos/{r["name"]}/branches'
        ):
            branch_name = b["displayId"]
            if matches(branch_name, branch_pattern):
                commits = get_all(
                    f'/rest/api/1.0/projects/{r["key"]}/repos/{r["name"]}/commits',
                    {"until": branch_name},
                    1,
                )
                if commits:
                    commit_datetime = datetime.fromtimestamp(
                        commits[0]["authorTimestamp"] / 1000
                    )
                    if young_enough(commit_datetime, max_head_commit_age):
                        branches[branch_name] = commit_datetime

        if branches:
            r = dict(r, branches=branches)
            logging.debug("Including repo: %s", r)
            return r

        logging.debug("Excluding repo: %s", r)
        return None

    for repo in pmap_unordered(inner, repos, processes=30):
        if repo:
            yield repo


def find_repos(project_pattern, repo_pattern):
    for p in get_projects():
        key = p["key"]
        if matches(key, project_pattern):
            for r in get_repos(p["key"]):
                if matches(r["name"], repo_pattern):
                    repo = {
                        "key": key,
                        "name": r["name"],
                        "cloneUrl": get_in(r, ["links", "clone", 0, "href"])
                        # 'cloneUrl': r['links']['clone'][0]['href'],
                    }
                    logging.debug("Found repo: %s", repo)
                    yield repo


def get_repo_info(
    project_pattern,
    repo_pattern,
    branch_pattern,
    tag_pattern,
    max_head_commit_age,
    **kwargs,
):
    # TODO: Add tag info
    repos = find_repos(project_pattern, repo_pattern)

    updated_repos = {}
    for repo in complement_and_filter_repos(repos, branch_pattern, max_head_commit_age):
        for branch in repo["branches"].keys():
            updated_repos[(repo["key"], repo["name"], branch)] = repo
    return updated_repos


def get_job_info(project_pattern, repo_pattern, branch_pattern, tag_pattern, **kwargs):
    # TODO: Add tag info
    jobs = {}
    for job in get_jobs():
        job_name = job["name"]
        try:
            key, name, branch = job_name.split("_")  # Is branch sometimes also tag?
            if (
                matches(key, project_pattern)
                and matches(name, repo_pattern)
                and matches(branch, branch_pattern)
            ):
                job = {"name": job_name, "url": job["url"], "repo": (key, name, branch)}
                jobs[(key, name, branch)] = job
                logging.debug("Including job: %s", job)
            else:
                logging.debug("Excluding job: %s", job_name)
        except ValueError:
            logging.debug("Excluding job: %s", job_name)
    return jobs


def synchronize(cfg):
    repos = get_repo_info(**cfg)
    jobs = get_job_info(**cfg)
    for job in jobs.values():
        if job["repo"] in repos:
            logging.info("Updating job: %s", job)
        else:
            logging.info("Deleting job: %s", job)

    for k, repo in repos.items():
        if k not in jobs:
            logging.info("Creating job: %s", repo)


if __name__ == "__main__":
    from timeit import default_timer as timer

    start = timer()

    cfg = {
        "project_pattern": "ESAUTO",
        "repo_pattern": ".*",
        "branch_pattern": "develop|release.*",
        "tag_pattern": [],
        "max_head_commit_age": 30,
    }

    synchronize(cfg)

    from timeit import default_timer as timer

    end = timer()
    print(end - start)



#!/usr/bin/env python3
import argparse
import re
from sys import argv

from vang.bitbucket.get_clone_urls import get_clone_urls
from vang.bitbucket.get_projects import get_projects


def get_clone_urls_grep(patterns, command=False):
    return get_clone_urls(
        [
            p["key"]
            for p in get_projects()
            if any((re.match(r"{}".format(pattern), p["key"]) for pattern in patterns))
        ],
        command,
    )


def main(patterns, command):
    for clone_dir, project, repo, clone_url in get_clone_urls_grep(patterns, command):
        print(clone_url)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Get clone urls for project patterns from Bitbucket"
    )
    parser.add_argument("patterns", nargs="+", help="The patterns")
    parser.add_argument(
        "-c", "--command", help="Print as clone commands", action="store_true"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import socket
from os import system


def get_ip_address():  # pragma: no cover
    """
    Returns the ip address.
    """
    return [
        (s.connect(("8.8.8.8", 53)), s.getsockname()[0], s.close())
        for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]
    ][0][1]


def main():
    ip_address = get_ip_address()
    system(f'echo "{ip_address}\\c" | pbcopy')
    print(ip_address)


if __name__ == "__main__":  # pragma: no cover
    main()



#!/usr/bin/env python3
import argparse
from datetime import datetime
from fnmatch import fnmatch
from json import dumps, loads
from os import makedirs
from os import walk
from os.path import dirname, relpath
from sys import argv
from zipfile import ZipFile


def create_zip(the_entries, output_file):
    makedirs(dirname(output_file), exist_ok=True)

    the_zip_file = output_file.replace(
        "#timestamp#", datetime.now().strftime("%Y%m%dT%H%M%S")
    )
    with ZipFile(the_zip_file, "w") as z:
        for base_path, rel_path, flatten in the_entries:
            file_name = f"{base_path}/{rel_path}"
            arc_name = rel_path.split("/")[-1] if flatten else rel_path
            z.write(file_name, arc_name)
    return the_zip_file


def has_match(name, patterns):
    return any([fnmatch(name, pattern) for pattern in patterns])


def get_patterns(key, default, **kwargs):
    if key not in kwargs:
        return default
    return [p.split("->")[0] for p in kwargs[key]]


def is_included(dir_path, file_name, path):
    rel_path = relpath(f"{dir_path}/{file_name}", path)
    return has_match("includes", get_patterns("includes", [".*"])) and not has_match(
        rel_path, get_patterns("excludes", [])
    )


def get_entries(dirs):
    return [
        (d["path"], relpath(f"{dir_path}/{file_name}", d["path"]), d["flatten"])
        for d in dirs
        for dir_path, dir_names, file_names in walk(d["path"])
        for file_name in file_names
        if is_included(dir_path, file_name, d["path"])
    ]


def load_config(config):
    with open(config, "rt", encoding="utf-8") as cfg:
        return loads(cfg.read())


def pzip(config):
    cfg = load_config(config)
    the_entries = get_entries(cfg["dirs"])
    the_zip_file = create_zip(the_entries, cfg["output_file"])
    return the_zip_file, the_entries


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Zip as configured in specified config file."
    )
    parser.add_argument(
        "config",
        help=dumps(
            {
                "dirs": [
                    {
                        "path": "/Users/me/.m2/repository/release/com/foo/bar",
                        "flatten": True,
                        "includes": [
                            "business*/**/*.jar",
                            "war.es/1.0.1-SNAPSHOT/war.es-1.0.1-SNAPSHOT.war",
                        ],
                        "excludes": ["**/*javadoc*", "**/*sources*"],
                    }
                ],
                "output_file": "./releases/release-#timestamp#.zip",
            }
        ),
    )
    return parser.parse_args(args)


def main(config):
    zip_file, entries = pzip(config)
    print(f"Zip file: {zip_file}")


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import re
from os import environ
from sys import argv

from requests import get, post


# Found no API for listing views so scraping the web page...
def list_views(url, username, password, verify_certificate):
    response = get(url=url, auth=(username, password), verify=verify_certificate)
    return {r for r in re.findall(r'href="/view/([^/]+)/', response.text, re.MULTILINE)}


def get_view_xml(name, job_names=tuple()):
    job_names_xml = "".join([f"<string>{n}</string>" for n in job_names])
    return f"""<hudson.model.ListView>
        <name>{name}</name>
        <filterExecutors>false</filterExecutors>
        <filterQueue>false</filterQueue>
        <properties class="hudson.model.View$PropertyList"/>
        <jobNames>{job_names_xml}</jobNames>
        <jobFilters/>
        <columns>
            <hudson.views.StatusColumn/>
            <hudson.views.WeatherColumn/>
            <hudson.views.JobColumn/>
            <hudson.views.LastSuccessColumn/>
            <hudson.views.LastFailureColumn/>
            <hudson.views.LastDurationColumn/>
            <hudson.views.BuildButtonColumn/>
        </columns>
        <recurse>false</recurse>
    </hudson.model.ListView>"""


def get_view(name, url, username, password, verify_certificate):
    uri = f"/view/{name}/config.xml"
    return get(url=f"{url}{uri}", auth=(username, password), verify=verify_certificate)


def view_exists(name, url, username, password, verify_certificate):
    return (
        get_view(name, url, username, password, verify_certificate).status_code == 200
    )


def do_post(uri, url, username, password, verify_certificate, data=None):
    return post(
        url=f"{url}{uri}",
        data=data if data else None,
        headers={"Content-Type": "text/xml"},
        auth=(username, password),
        verify=verify_certificate,
    )


def create_view(name, job_names, url, username, password, verify_certificate):
    uri = f"/createView?name={name}"
    return do_post(
        uri, url, username, password, verify_certificate, get_view_xml(name, job_names)
    ).status_code


def delete_view(name, url, username, password, verify_certificate):
    uri = f"/view/{name}/doDelete"
    return do_post(uri, url, username, password, verify_certificate).status_code


def update_view(name, job_names, url, username, password, verify_certificate):
    uri = f"/view/{name}/config.xml"
    return do_post(
        uri, url, username, password, verify_certificate, get_view_xml(name, job_names)
    ).status_code


def set_view(name, job_names, url, username, password, verify_certificate):
    params = (url, username, password, verify_certificate)
    if view_exists(name, *params):
        return update_view(name, job_names, *params)
    return create_view(name, job_names, *params)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Set (Create/Update Jenkins view with jobs"
    )
    parser.add_argument(
        "name",
        help="Jenkins view name",
    )
    parser.add_argument(
        "job_names",
        nargs="+",
        help="Jenkins job names",
    )
    parser.add_argument(
        "--url", "-u", help="Jenkins url", default=environ.get("JENKINS_REST_URL", None)
    )
    parser.add_argument(
        "--username",
        "-n",
        help="Jenkins username",
        default=environ.get("JENKINS_USERNAME", None),
    ),
    parser.add_argument(
        "--password",
        "-p",
        help="Jenkins password",
        default=environ.get("JENKINS_PASSWORD", None),
    )
    parser.add_argument(
        "--verify_certificate",
        "-v",
        help="Verify Jenkins certificate",
        default=not environ.get("JENKINS_IGNORE_CERTIFICATE", None),
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    response_code = set_view(**parse_args(argv[1:]).__dict__)
    print(response_code)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.get_branches import get_branches
from vang.bitbucket.utils import get_repo_specs


def has_branch(repo_specs, branch):
    for spec in repo_specs:
        branches = [
            b["displayId"] for spec, bs in get_branches((spec,), branch) for b in bs
        ]
        yield spec, branch in branches


def main(
    branch, only_has=True, only_not_has=False, dirs=None, repos=None, projects=None
):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, has in has_branch(specs, branch):
        if only_has:
            if has:
                print(f"{spec[0]}/{spec[1]}")
        elif only_not_has:
            if not has:
                print(f"{spec[0]}/{spec[1]}")
        else:
            print(f"{spec[0]}/{spec[1]}, {branch}: {has}")


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Check repository branches in Bitbucket"
    )
    parser.add_argument("branch", help="The branch to check")
    filter_group = parser.add_mutually_exclusive_group()
    filter_group.add_argument(
        "-o",
        "--only_has",
        action="store_true",
        help="Print only repos that has the branch.",
    )
    filter_group.add_argument(
        "-n",
        "--only_not_has",
        action="store_true",
        help="Print only repos that not has the branch.",
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Git directories to extract repo information from",
    )
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import environ
from sys import argv

from vang.jenkins.api import call

FAILURE = "FAILURE"
SUCCESS = "SUCCESS"
NOT_BUILT = "NOT_BUILT"
UNKNOWN = "UNKNOWN"


def map_color(color):
    if color == "blue":
        return SUCCESS
    if color == "red":
        return FAILURE
    if color == "notbuilt":
        return NOT_BUILT
    return UNKNOWN


def get_jobs(
    statuses=(FAILURE, SUCCESS, NOT_BUILT, UNKNOWN),
    only_names=False,
    url=environ.get("JENKINS_REST_URL", None),
    username=environ.get("JENKINS_USERNAME", None),
    password=environ.get("JENKINS_PASSWORD", None),
    verify_certificate=not environ.get("JENKINS_IGNORE_CERTIFICATE", None),
    view=None,
):
    jobs = call(
        f"/view/{view}/api/json" if view else "/api/json",
        rest_url=url,
        username=username,
        password=password,
        verify_certificate=verify_certificate,
    )["jobs"]
    return [
        job["name"] if only_names else job
        for job in jobs
        if map_color(job.get("color", None)) in statuses
    ]


def parse_args(args):
    parser = argparse.ArgumentParser(description="Get Jenkins jobs")
    parser.add_argument(
        "-n", "--only_names", action="store_true", help="Get only job names"
    )
    parser.add_argument("-v", "--view", default=None, help="Get only jobs in view")
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        "-f",
        "--only_failures",
        action="store_true",
        help="Only failure jobs",
    )
    group.add_argument(
        "-s",
        "--only_successes",
        action="store_true",
        help="Only success jobs",
    )
    group.add_argument(
        "-b",
        "--only_not_built",
        action="store_true",
        help="Only not built jobs",
    )
    group.add_argument(
        "-u",
        "--only_unknown",
        action="store_true",
        help="Only unknown status jobs",
    )
    return parser.parse_args(args)


def main(only_failures, only_successes, only_names, only_not_built, only_unknown, view):
    if only_failures:
        job_statuses = [FAILURE]
    elif only_successes:
        job_statuses = [SUCCESS]
    elif only_not_built:
        job_statuses = [NOT_BUILT]
    elif only_unknown:
        job_statuses = [UNKNOWN]
    else:
        job_statuses = [FAILURE, SUCCESS, NOT_BUILT, UNKNOWN]

    for a_job in get_jobs(job_statuses, only_names, view=view):
        print(a_job)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



from json import loads


def get_definition_name(project, repo, branch):
    return f"{project}_{repo}_{branch}"


def get_definition(template, replacements={}, additions={}):
    with open(template, "rt", encoding="utf-8") as template_file:
        request = template_file.read()
        for key, value in replacements.items():
            request = request.replace(f"##{key}##", value)
        return {**loads(request), **additions}



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.tfs.api import call


def delete_build_definition(project, definition_id):
    organisation, project = project.split("/")
    return call(
        f"/{organisation}/{project}/_apis/build/definitions/{definition_id}?api-version=3.2",
        method="DELETE",
        only_response_code=True,
    )


def main(project, definition_ids):
    for definition_id in definition_ids:
        response_code = delete_build_definition(project, definition_id)
        print(definition_id, response_code)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Delete TFS build definitions")
    parser.add_argument("project", help="TFS projects, e.g organisation/project")
    parser.add_argument(
        "definition_ids", nargs="+", help="The TFS build definitions id, e.g. 1234"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import logging
from json import loads
from os import environ
from os.path import basename
from sys import argv
from typing import Dict, Tuple

from requests import post, get

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = "https://dev.azure.com"


def get_definition_id(
    token: str,
    organization: str,
    project: str,
    build_definition_name: str,
    verify_certificate: bool = True,
    api_version: str = "6.1-preview.7",
):
    url = f"{base_url}/{organization}/{project}/_apis/build/definitions?name={build_definition_name}&api-version={api_version}"
    params = {"url": url, "auth": ("", token), "verify": verify_certificate}
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = get(**params)
    logger.info(f"response.status_code: {response.status_code}")
    logger.info(f"response.text: {response.text}")
    response.raise_for_status()
    return loads(response.text)["value"][0]["id"]


def create_json(definition_id: int, branch: str) -> Dict:
    return {"definition": {"id": definition_id}, "sourceBranch": f"refs/heads/{branch}"}


def queue_build(
    token: str,
    organization: str,
    project: str,
    build_definition_name: str,
    branch: str,
    verify_certificate: bool = True,
    api_version: str = "6.1-preview.6",
) -> Tuple[int, Dict]:
    definition_id = get_definition_id(
        token, organization, project, build_definition_name, verify_certificate
    )
    url = f"{base_url}/{organization}/{project}/_apis/build/builds?api-version={api_version}"
    params = {
        "json": create_json(definition_id, branch),
        "url": url,
        "auth": ("", token),
        "verify": verify_certificate,
    }
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = post(**params)
    logger.info(f"response.status_code: {response.status_code}")
    logger.info(f"response.text: {response.text}")
    response.raise_for_status()
    return response.status_code, loads(response.text)


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(description="Queue a build")
    parser.add_argument(
        "--token",
        default=environ.get("AZDO_TOKEN", ""),
        help="The Azure DevOps authorisation token",
    )
    parser.add_argument(
        "--organisation",
        default=environ.get("AZDO_ORGANISATION", ""),
        help="The Azure DevOps organisation",
    )
    parser.add_argument(
        "--project",
        default=environ.get("AZDO_PROJECT", ""),
        help="The Azure DevOps project",
    )
    parser.add_argument(
        "build_definition_name", help="The Azure DevOps build definition name to queue"
    )
    parser.add_argument("branch", help="The Azure DevOps repo branch to build")
    parser.add_argument(
        "-au",
        "--azure_devops_url",
        default="https://dev.azure.com",
        help="The Azure DevOps REST API base url",
    )

    return parser.parse_args(args)


def main(
    token: str,
    organisation: str,
    project: str,
    build_definition_name: str,
    branch: str,
    azure_devops_url: str,
) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    status_code, build = queue_build(
        token, organisation, project, build_definition_name, branch
    )
    print(status_code)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import logging
from json import loads
from os import environ
from os.path import basename
from sys import argv
from typing import Dict, Tuple

from requests import post

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = "https://dev.azure.com"


def create_json(repository_name: str) -> Dict:
    return {
        "triggers": [
            {
                "branchFilters": [],
                "pathFilters": [],
                "settingsSourceType": 2,
                "batchChanges": False,
                "maxConcurrentBuildsPerBranch": 1,
                "triggerType": "continuousIntegration",
            }
        ],
        "process": {"yamlFilename": "azure-pipelines.yml", "type": 2},
        "repository": {"name": repository_name, "type": "TfsGit"},
        "queue": {"id": 8},
        "name": repository_name,
    }


def create_build_definition(
    token: str,
    organization: str,
    project: str,
    repository_name: str,
    verify_certificate: bool = True,
    api_version: str = "6.1-preview.7",
) -> Tuple[int, Dict]:
    url = f"{base_url}/{organization}/{project}/_apis/build/definitions?api-version={api_version}"
    params = {
        "json": create_json(repository_name),
        "url": url,
        "auth": ("", token),
        "verify": verify_certificate,
    }
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = post(**params)
    logger.info(f"response.status_code: {response.status_code}")
    logger.info(f"response.text: {response.text}")
    response.raise_for_status()
    return response.status_code, loads(response.text)


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(description="Create a build_definition")
    parser.add_argument(
        "--token",
        default=environ.get("AZDO_TOKEN", ""),
        help="The Azure DevOps authorisation token",
    )
    parser.add_argument(
        "--organisation",
        default=environ.get("AZDO_ORGANISATION", ""),
        help="The Azure DevOps organisation",
    )
    parser.add_argument(
        "--project",
        default=environ.get("AZDO_PROJECT", ""),
        help="The Azure DevOps project",
    )
    parser.add_argument(
        "repository_name",
        help="The Azure DevOps build definition to create, should be the name of an existing repo",
    )
    parser.add_argument(
        "-au",
        "--azure_devops_url",
        default="https://dev.azure.com",
        help="The Azure DevOps REST API base url",
    )

    return parser.parse_args(args)


def main(
    token: str,
    organisation: str,
    project: str,
    repository_name: str,
    azure_devops_url: str,
) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    status_code, build_definition = create_build_definition(
        token, organisation, project, repository_name
    )
    print(status_code)
    # pprint(build_definition)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from argparse import ArgumentParser
from os import makedirs
from re import match

from vang.pio.shell import run_command


def create_patch(repo, since_tag, tag, output):
    patch_dir = f"{output}/{tag}"
    makedirs(patch_dir)
    cmd = " ".join(
        [
            "git log -p --reverse --pretty=email --full-index --binary",
            "--stat",
            "-m",
            "--first-parent" f'{since_tag + ".." if since_tag else ""}{tag}',
            f"> {patch_dir}/tag.patch",
        ]
    )
    print(cmd)
    rc, out = run_command(cmd, True, repo)
    return tag, patch_dir, out


def apply_patch(repo, tag, patch_dir):
    print("Apply patch", tag, patch_dir)
    return [
        run_command(cmd, True, repo)
        for cmd in [
            f"git apply {patch_dir}/*",
            "git add --all",
            f"git commit -m {tag}",
            f"git tag -a {tag} -m {tag}",
        ]
    ]


def get_tags(repo, tag_pattern):
    return [
        tag
        for tag in run_command("git tag", True, repo)[1].split("\n")
        if match(r"{}".format(tag_pattern), tag)
    ]


def get_unpatched_tags(patchs_tags, applied_tags):
    return [p for p in patchs_tags if p not in applied_tags]


def is_valid(patchs_tags, applied_tags):
    return all([p == a for p, a in zip(patchs_tags, applied_tags)]) and not len(
        applied_tags
    ) > len(patchs_tags)


def main(patch_repo, tag_pattern, output, apply_repo):
    patchs_tags = get_tags(patch_repo, tag_pattern)
    applied_tags = get_tags(apply_repo, tag_pattern)
    if is_valid(patchs_tags, applied_tags):
        unpatched_tags = get_unpatched_tags(patchs_tags, applied_tags)
        patch_ranges = zip([None] + unpatched_tags, unpatched_tags)
        created_patches = [
            create_patch(patch_repo, since_tag, until_tag, output)
            for since_tag, until_tag in patch_ranges
        ]
        applied_patches = [
            apply_patch(apply_repo, tag, patch_dir)
            for tag, patch_dir, out in created_patches
        ]
        return applied_patches
    else:
        raise ValueError("Tags are not valid.")


def parse_args(args):
    parser = ArgumentParser(
        description="Create patches of tags and applies, "
        + "commits and tags them in another repo."
    )
    parser.add_argument("tag_pattern", help="A tag pattern.")
    parser.add_argument("apply_repo", help="The repo to apply patches to.")
    parser.add_argument(
        "-p",
        "--patch_repo",
        help="The repo to patch from.",
        default=".",
    )
    parser.add_argument(
        "-o",
        "--output",
        help="A directory to put patches in.",
        default="./patch",
    )
    return parser.parse_args(args)



#!/usr/bin/env python3
import argparse
from filecmp import cmp
from os import makedirs, remove, walk
from os.path import exists
from shutil import copyfile, rmtree
from sys import argv


def add(dest_dir, ignore_sub_paths, source_dir):
    adds = []
    for path, dirs, files in walk(source_dir):
        sub_path = path[len(source_dir) :]
        if not any([sub_path.startswith(ignore) for ignore in ignore_sub_paths]):
            dest_path = dest_dir + sub_path
            makedirs(dest_path, exist_ok=True)
            for file in files:
                source_file = f"{path}/{file}"
                dest_file = f"{dest_path}/{file}"
                if exists(dest_file):
                    if not cmp(source_file, dest_file):
                        copyfile(source_file, dest_file)
                        adds.append(f"Updated from {source_file} to {dest_file}")
                        print(adds[-1])
                else:
                    copyfile(source_file, dest_file)
                    adds.append(f"Added from {source_file} to {dest_file}")
                    print(adds[-1])
    return adds


def delete(dest_dir, ignore_sub_paths, source_dir):
    deletes = []
    for path, dirs, files in walk(dest_dir):
        sub_path = path[len(dest_dir) :]
        if not any([sub_path.startswith(ignore) for ignore in ignore_sub_paths]):
            source_path = source_dir + sub_path
            if not exists(source_path):
                rmtree(path)
                deletes.append(f"Removed dir {path}")
                print(deletes[-1])
            else:
                for file in files:
                    source_file = f"{source_path}/{file}"
                    dest_file = f"{path}/{file}"
                    if not exists(source_file):
                        remove(dest_file)
                        deletes.append(f"Removed file {dest_file}")
                        print(deletes[-1])
    return deletes


def synchronize_dirs(source_dir, dest_dir, ignore_sub_paths=("/.git",)):
    """
    Makes dest_dir look exactly as as source_dir
    with exception of ignore_sub_paths
    :param source_dir: path as string
    :param dest_dir: path as string
    :param ignore_sub_paths: ignored sub paths as list of string
    :return: List describing synchronizations done
    """
    adds = add(dest_dir, ignore_sub_paths, source_dir)
    deletes = delete(dest_dir, ignore_sub_paths, source_dir)
    return adds + deletes


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Makes dest_dir look exactly as as source_dir "
        "with exception of ignore_sub_paths"
    )
    parser.add_argument("source_dir", help="path as string")
    parser.add_argument("dest_dir", help="path as string")
    parser.add_argument(
        "-i",
        "--ignore_sub_paths",
        nargs="+",
        help="ignored sub paths as list of string",
        default=["/.git"],
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    synchronize_dirs(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from os import environ

from vang.jenkins.copy_jobs import copy_jobs

# This script can be used to copy jobs between Jenkins instances, possibly with modifications.
# With a minor modification, it can also be used to update jobs on a Jenkins instance.

# Can be an iterable with pairs of old and new, or a function
# that takes the from_job configuration as parameter and returns the updated configuration.
# Will be used if no job specific replacement has been specified.
replacements = [
    ("old_foo", "new_foo"),
    ("old_bar", "new_bar"),
]

# An iterable with strings and/or
# a pair of strings with job names and/or
# a pair of string with job names and job specific replacements (see replacements).
#
# If only a string, the name of from_job and to_job will be the same.
# If a pair of strings, the first element is the name of the from_job and the second will be the name of the clone_job,
# i.e. this can be used to rename the clone.
# A third element can be used to specify job specific replacements, i.e. it overrides the default replacements.
jobs = [
    ("from_job_1", "to_job_1"),
    ("from_job_2", "to_job_2", lambda x: x),
    ("from_job_3", "to_job_3", [("old_baz", "new_baz")]),
]

# Spec of the Jenkins instance to copy from
from_jenkins_spec = {
    "url": environ.get("JENKINS_REST_URL", None),
    "username": environ["JENKINS_USERNAME"],
    "password": environ["JENKINS_PASSWORD"],
    "verify_certificate": not environ.get("JENKINS_IGNORE_CERTIFICATE", None),
}

# Spec of the Jenkins instance to copy to
to_jenkins_spec = dict(from_jenkins_spec, url="<to_jenkins_url>")

# Call the copy_jobs function to perform the actual replacement
copy_jobs(jobs, from_jenkins_spec, to_jenkins_spec, replacements)



#!/usr/bin/env python3

import argparse
from pprint import pprint
from re import search
from sys import argv


def get_dependants(dependency_tree_file, dependency):
    dependency_dict = {}
    with open(dependency_tree_file, "rt") as file:
        previous_line = ""
        artifact = None
        for line in file.readlines():
            if previous_line.startswith("[INFO] --- maven-dependency-plugin"):
                artifact = line.split(" ")[1].strip()
            elif line.strip() == "[INFO]":
                artifact = None
            elif artifact and search(r"{}".format(dependency), line):
                key = line.rsplit(" ", maxsplit=1)[1].strip()
                dependants = dependency_dict.get(
                    key, {"direct": set(), "transitive": set()}
                )
                dependency_dict[key] = dependants
                dependants["transitive" if "|" in line else "direct"].add(artifact)
            previous_line = line
    return dependency_dict


def main(dependency_tree_file, dependency, only_direct, only_transitive):
    dependants = get_dependants(dependency_tree_file, dependency)
    key = "direct" if only_direct else "transitive" if only_transitive else None

    if key:
        d = {}
        for k, v in dependants.items():
            d[k] = v[key]
        dependants = d

    pprint(dependants)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Get direct and transitive dependants from provided dependency_tree_file, "
        "e.g. ./get_dependants.py ./dependency_tree.txt org.slf4j:slf4j-api"
    )
    parser.add_argument(
        "dependency_tree_file",
        help="Dependency tree file (created with e.g. mvn dependency:tree > dependency_tree.txt",
    )
    parser.add_argument(
        "dependency",
        help="The dependency to analyse, e.g. logback. "
        "Can be a fully qualified dependency, e.g. org.slf4j:slf4j-api:jar:1.7.7:compile, "
        "or a substring of a dependency, e.g. slf4j-api:jar:1.7.7, "
        'or a quoted regular expression substring, e.g. "slf4j.*1.7.7".',
    )
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        "-d", "--only_direct", help="Print only direct dependants", action="store_true"
    )
    group.add_argument(
        "-t",
        "--only_transitive",
        help="Print only transitive dependants",
        action="store_true",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from random import randint
from sys import argv

from vang.pio.shell import run_command


def get_emails(clone_dir, committer=False):
    log_format = "%ce" if committer else "%ae"
    rc, output = run_command(
        f'git log --format="{log_format}"',
        True,
        clone_dir,
    )
    return set(output.split("\n"))


def filter_branch(clone_dir, new_name, new_email, old_email=None):
    filter_branch_command = (
        f"""git filter-branch --force --env-filter '
        if [ "$GIT_COMMITTER_EMAIL" = "{old_email}" ]
        then
            export GIT_COMMITTER_NAME="{new_name}"
            export GIT_COMMITTER_EMAIL="{new_email}"
        fi
        if [ "$GIT_AUTHOR_EMAIL" = "{old_email}" ]
        then
            export GIT_AUTHOR_NAME="{new_name}"
            export GIT_AUTHOR_EMAIL="{new_email}"
        fi
    ' --tag-name-filter cat -- --branches --tags
    """
        if old_email
        else f"""git filter-branch --force --env-filter '
        export GIT_COMMITTER_NAME="{new_name}"
        export GIT_COMMITTER_EMAIL="{new_email}"
        export GIT_AUTHOR_NAME="{new_name}"
        export GIT_AUTHOR_EMAIL="{new_email}"
    ' --tag-name-filter cat -- --branches --tags
    """
    )
    return run_command(filter_branch_command, True, clone_dir)


def create_random_name_and_email():
    name = f"dev.{randint(1000000, 9999999)}"
    return name, f"{name}@it.com"


def main(clone_dir, distinct=False):
    if distinct:
        for email in set(get_emails(clone_dir) | get_emails(clone_dir, True)):
            new_name, new_email = create_random_name_and_email()
            print(f"Filtering: {email} -> {new_email}")
            filter_branch(clone_dir, new_name, new_email, email)
    else:
        new_name, new_email = create_random_name_and_email()
        print(f"Filtering: * -> {new_email}")
        filter_branch(clone_dir, new_name, new_email)

    print("#" * 80)
    print(
        "emails after filtering:",
        f"{set(get_emails(clone_dir) | get_emails(clone_dir, True))}",
    )


def parse_args(args):
    parser = ArgumentParser(
        description="Randomize author and commit names for all git commits."
    )
    parser.add_argument("-c", "--clone_dir", help="The git repo directory", default=".")
    parser.add_argument(
        "-d", "--distinct", help="If distinct mapping", action="store_true"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    sure = input("Sure? (y/n): ") == "y"
    if sure:
        main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import logging
from json import loads
from os import environ
from os.path import basename
from pprint import pprint
from sys import argv
from typing import Iterable

from requests import get

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = "https://dev.azure.com"


def list_builds(
    token: str,
    organisation: str,
    project: str,
    verify_certificate: bool = True,
    api_version: str = "6.1-preview.6",
) -> Iterable:
    url = f"{base_url}/{organisation}/{project}/_apis/build/builds?api-version={api_version}"
    params = {"url": url, "auth": ("", token), "verify": verify_certificate}
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = get(**params)
    logger.info(f"response.status_code: {response.status_code}")
    logger.info(f"response.text: {response.text}")
    response.raise_for_status()

    builds = loads(response.text)
    return builds["value"]


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(description="List builds")
    parser.add_argument(
        "--token",
        default=environ.get("AZDO_TOKEN", ""),
        help="The Azure DevOps authorisation token",
    )
    parser.add_argument(
        "--organisation",
        default=environ.get("AZDO_ORGANISATION", ""),
        help="The Azure DevOps organisation",
    )
    parser.add_argument(
        "--project",
        default=environ.get("AZDO_PROJECT", ""),
        help="The Azure DevOps project",
    )
    parser.add_argument(
        "-au",
        "--azure_devops_url",
        default="https://dev.azure.com",
        help="The Azure DevOps REST API base url",
    )

    return parser.parse_args(args)


def main(
    token: str, organisation: str, project: str, azure_devops_url: str
) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    for p in list_builds(token, organisation, project):
        pprint(p)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from argparse import ArgumentParser
from os import environ
from pprint import pprint
from sys import argv

from requests import get


def get_repositories(url, username, password):
    url = f"{url}/service/rest/v1/repositories"
    response = get(url, auth=(username, password))
    if response.status_code != 200:
        raise OSError(f"{response.status_code}, {response.content}")
    return response.json()


def parse_args(args):
    parser = ArgumentParser(description="Get repositories")
    parser.add_argument(
        "-l",
        "--url",
        default=environ.get("NEXUS3_REST_URL", None),
        help="Nexus3 url, e.g. http://nexus_host:8080",
    )
    parser.add_argument(
        "-u",
        "--username",
        default=environ.get("NEXUS3_USERNAME", None),
        help="Nexus3 username",
    )
    parser.add_argument(
        "-p",
        "--password",
        default=environ.get("NEXUS3_PASSWORD", None),
        help="Nexus3 password",
    )
    return parser.parse_args(args)


def main(url, username, password):
    pprint(get_repositories(url, username, password))


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.tfs.api import call


def delete_repo(repo):
    organisation, project, name = repo.split("/")
    repo_id = call(
        f"/{organisation}/{project}/_apis/git/repositories/{name}?api-version=3.2",
        method="GET",
    )["id"]
    return call(
        f"/{organisation}/{project}/_apis/git/repositories/{repo_id}?api-version=3.2",
        method="DELETE",
        only_response_code=True,
    )


def main(repo):
    response = delete_repo(repo)
    print(response)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Delete TFS repository")
    parser.add_argument(
        "repo", help="The TFS repository to delete, e.g. organisation/project/repo1"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from os import walk
from pprint import pprint
from re import fullmatch
from sys import argv


def is_excluded(file, excluded):
    return any([fullmatch(ex, file) for ex in excluded])


def is_included(file, included):
    return any([fullmatch(ex, file) for ex in included])


def get_files(root_dir, excluded=(), included=(".*",)):
    for root, dirs, files in walk(root_dir):
        for f in files:
            if is_included(f, included) and not is_excluded(f, excluded):
                yield root, f


def count_words(line):
    n = 0
    for s in line.split(" "):
        if s.strip():
            n += 1
    return n


def count_letters(line):
    return len(line.strip())


def count(root, file):
    line_count = 0
    word_count = 0
    letter_count = 0
    with open(f"{root}/{file}", "rt", encoding="utf-8") as f:
        for line in f:
            if line.strip():
                line_count += 1
                word_count += count_words(line)
                letter_count += count_letters(line)
    return line_count, word_count, letter_count


def count_all(dirs=(".",), excluded=(), included=(".*",)):
    total_files = 0
    total_lines = 0
    total_words = 0
    total_letters = 0

    for d in dirs:
        for root, file in get_files(d, excluded, included):
            total_files += 1
            line_count, word_count, letter_count = count(root, file)
            total_lines += line_count
            total_words += word_count
            total_letters += letter_count

    return {
        "files": total_files,
        "lines": total_lines,
        "words": total_words,
        "letters": total_letters,
    }


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Count files, lines, words and letters."
    )
    parser.add_argument(
        "-d", "--dirs", nargs="*", default=["."], help="Directories to count in"
    )
    parser.add_argument(
        "-e",
        "--excluded",
        nargs="*",
        default=[],
        help="File name exclusion patterns, e.g .*Test\\..* .*IT\\..*",
    )
    parser.add_argument(
        "-i",
        "--included",
        nargs="*",
        default=[".*"],
        help="File name inclusion patterns, e.g .*\\.groovy .*\\.java .*\\.py",
    )
    return parser.parse_args(args)


def main(dirs=(".",), excluded=(), included=(".*",)):
    result = count_all(dirs, excluded=excluded, included=included)
    pprint(result)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)

# code_dirs = ['/Users/magnus/git/rsimulator/rsimulator-camel-direct',
#              '/Users/magnus/git/rsimulator/rsimulator-core',
#              '/Users/magnus/git/rsimulator/rsimulator-cxf-rt-transport']
# code_dirs = ['/Users/magnus/git/rsimulator']
#
# result = count_all(
#     code_dirs,
#     # excluded=('.*Test\..*', '.*IT\..*', 'test.*'),
#     included=('.*\.groovy', '.*\.java', '.*\.kt', '.*\.py', 'Jenkinsfile'))
# pprint(result)
